package benchmark;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import org.jfree.data.xy.XYSeries;

import benchmark.graphcoloring.URGCGenerator;
import benchmark.meetingscheduling.URMeetingScheduling;
import benchmark.nqueen.NQueenGenerator;
import benchmark.random.RBCDisCSPGenerator;
import benchmark.random.RBDisCSPGenerator;
import benchmark.rmsp.RandomMSP;
import benchmark.sensornetwork.SensorDCSPGenerator;
import util.Commands;
import util.Logger;
import util.Tools;


/**
 * A class for printing results obtained by algorithms after doing experimentations 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */

public class ResultPrinter {

	private String[] algorithms; 
	private String[] orderings; 
	private String[] keys = {Commands.MSGs, Commands.NCCCs, Commands.SOLVABLE};//, Commands.GNCCCs, Commands.CCs
	private String fixKeys; // Fix keys
	private Data[] data;
	private int nbHeaderLine = 4;
	private File[] files = null;
	private final String scriptName = "myScript.gp";
	private String type = "";
	public static String gnuplot = "/usr/local/bin/gnuplot";
	public static String epstopdf = "epstopdf";

	public ResultPrinter(String[] algorithms, String[] orderings, int nbHeaderLine) {
		this.algorithms = algorithms;
		this.orderings = orderings;
		this.data = new Data[algorithms.length];
		for (int i = 0 ; i < this.data.length ; i++) {
			this.data[i] = new Data();
		}
		this.nbHeaderLine = nbHeaderLine;
	}

	public ResultPrinter(String[] algorithms, String[] orderings, String[] keys, String fixKeys, int nbHeaderLine) {
		this(algorithms, orderings, nbHeaderLine);
		this.keys = keys;
		this.fixKeys = fixKeys;
	}

	public ResultPrinter(String[] names) {
		this.files = new File[names.length];
		this.data = new Data[names.length];
		this.algorithms = new String[names.length];
		this.orderings = new String[names.length];
		for (int i = 0 ; i < names.length ; i++) {
			this.files[i] = new File(names[i]);
			try {
				BufferedReader br = new BufferedReader(new FileReader(names[i]));
				String line = br.readLine();
				String[] str = line.split(",");
				this.type = str[0];
				if (this.type.equals(Commands.RandomDisCSP)) {
					this.nbHeaderLine = RBDisCSPGenerator.nbHeaderLine;
					this.fixKeys = RBDisCSPGenerator.fixKeys;
				} else if (this.type.equals(Commands.GraphColoring)) {
					this.nbHeaderLine = URGCGenerator.nbHeaderLine;
					this.fixKeys = URGCGenerator.fixKeys;
				} else if (this.type.equals(Commands.NQueen)) {
					this.nbHeaderLine = NQueenGenerator.nbHeaderLine;
					this.fixKeys = NQueenGenerator.fixKeys;
				} else if (this.type.equals(Commands.ComplexRandomTKC)) {
					this.nbHeaderLine = RBCDisCSPGenerator.nbHeaderLine;
					this.fixKeys = RBCDisCSPGenerator.fixKeys;
				} else if (this.type.equals(Commands.GSensorDCSP) || this.type.equals(Commands.SensorDCSP)) {
					this.nbHeaderLine = SensorDCSPGenerator.nbHeaderLine;
					this.fixKeys = SensorDCSPGenerator.fixKeys;
				} else if (this.type.equals(Commands.MeetingScheduling)) {
					this.nbHeaderLine = URMeetingScheduling.nbHeaderLine;
					this.fixKeys = URMeetingScheduling.fixKeys;
				}else if (this.type.equals(Commands.RMSP)) {
					this.nbHeaderLine = RandomMSP.nbHeaderLine;
					this.fixKeys = RandomMSP.fixKeys;
				}// TODO add others types
				this.algorithms[i] = str[2];
				this.orderings[i] = str[4];
				br.close();
				this.data[i] = Data.fromFile(names[i], nbHeaderLine);
			} catch (Exception e) {}
		}
	}
	public void setGnuplot(String cmd){
		gnuplot = cmd;
	}
	/**
	 * @param keys
	 */
	public void plot(String[] keys) {
		this.setKeys(keys);
		this.plot();
	}

	/**
	 * @param keys
	 */
	public void setKeys(String[] keys) {
		this.keys = keys;
	}
	/**
	 * 
	 */
	public void plot() {
		String commType = Commands.FAST_COM;
		Logger results[] = new Logger[this.keys.length];
		for (int k = 0 ; k < this.keys.length ; k++) {
			results[k] = new Logger(this.files[0].getParent(), this.keys[k] + ".dat");
		}
		String line = this.type + ",";
		for (int p = 0 ; p < this.algorithms.length ; p++) {
			line += this.algorithms[p] + ", " + this.orderings[p] + ", ";
		}
		//print the header describing the experimentation on the .dat file 
		for (int k = 0 ; k < this.keys.length ; k++) {
			results[k].printToLogln("# " + line);
		}
		try {
			FileReader file = new FileReader(this.files[0]);
			BufferedReader br = new BufferedReader(file);
			br.readLine();
			line = br.readLine();
			for (int k = 0 ; k < this.keys.length ; k++) {
				results[k].printToLogln("# " + line);
			}
			line = br.readLine();
			String[] str = line.split(",");
			commType = str[3];
			for (int k = 0 ; k < this.keys.length ; k++) {
				results[k].printToLogln("# " + line);
			}
			br.close();
		} catch (IOException e) {}
		line = "" + fixKeys + " ";
		for (int k = 0 ; k < this.keys.length ; k++) {
			results[k].printToLog(line);
		}
		line = "";
		for (int p = 0 ; p < this.algorithms.length ; p++) {
			line += this.algorithms[p] + " ";
		}
		for (int k = 0 ; k < this.keys.length ; k++) {
			results[k].printToLogln(line);
		}
		for (int k = 0 ; k < this.keys.length ; k++) {
			for (int l = 0 ; l < this.data[0].getNbLine() ; l++) {
				for (int p = 0 ; p < this.algorithms.length ; p++) {
					line = "";
					Data d = this.data[p];
					if (p == 0) {
						int c = d.getKey(fixKeys);
						if (c == -1) {
							line += "- ";
						} else {
							line += d.getElement(l)[c] + " ";
						}
					}
					int c = d.getKey(this.keys[k]);
					if (c == -1) {
						line += "- ";
					} else {
						line += d.getElement(l)[c] + " ";
					}
					results[k].printToLog(line);
				}
				results[k].printToLogln("");
			}
			results[k].close();
		}
		ArrayList<String> paths = this.generateGPSript(this.files[0].getParent(), commType);
		this.plot(this.files[0].getParent(), paths);
	}

	public void setFixKeys(String fixKeys) {
		this.fixKeys = fixKeys;
	}

	public void readData(String directory, int nbHeaderLine) {
		int length = this.algorithms.length;
		this.data = new Data[length];
		for (int p = 0 ; p < length ; p++) {
			String fileName = directory + File.separator + this.algorithms[p] + "-" + this.orderings[p] + ".csv"; // //TODO
			this.data[p] = Data.fromFile(fileName, nbHeaderLine);
		}
	}

	@SuppressWarnings({"resource"})
	public ArrayList<String> printToGnuPlot(String directory, String subFolder, String commType) {
		int length = this.algorithms.length;
		this.readData(directory, this.nbHeaderLine);
		Logger results[] = new Logger[this.keys.length];
		for (int k = 0 ; k < this.keys.length ; k++) {
			results[k] = new Logger(directory + File.separator + subFolder, this.keys[k] + ".dat");
		}
		String line = "";
		for (int p = 0 ; p < length ; p++) {
			line += this.algorithms[p] + ":" + this.orderings[p] + "   ";
		}
		for (int k = 0 ; k < this.keys.length ; k++) {
			results[k].printToLogln("# " + line);
		}
		String path = directory + File.separator + this.algorithms[0] + "-" + this.orderings[0] + ".csv";
		FileReader file = null;
		try {
			file = new FileReader(path);
			BufferedReader br = new BufferedReader(file);
			line = br.readLine();
			line = br.readLine();
		} catch (IOException e) {}
		line = line.replaceAll(",", "  ");
		for (int k = 0 ; k < this.keys.length ; k++) {
			results[k].printToLogln("# " + line);
		}
		line = "" + fixKeys + " ";
		// for (int fk = 0; fk < this.fixKeys.length; fk++) {
		// line += "" + fixKeys[fk] + " ";
		// }
		for (int k = 0 ; k < this.keys.length ; k++) {
			results[k].printToLog(line);
		}
		line = "";
		for (int p = 0 ; p < length ; p++) { // line indicating the names of
			// algorithms
			// line += this.algorithms[p]+"-"+this.orderings[p]+" ";
			line += this.algorithms[p] + " ";
		}
		for (int k = 0 ; k < this.keys.length ; k++) {
			results[k].printToLogln(line);
		}
		for (int k = 0 ; k < this.keys.length ; k++) {
			for (int l = 0 ; l < this.data[0].getNbLine() ; l++) {
				for (int p = 0 ; p < length ; p++) {
					line = "";
					Data d = this.data[p];
					if (p == 0) {
						int c = d.getKey(fixKeys);
						if (c == -1) {
							line += "0 ";
						} else {
							line += d.getElement(l)[c] + " ";
						}
					}
					int c = d.getKey(this.keys[k]);
					if (c == -1) {
						line += "0 ";
					} else {
						line += d.getElement(l)[c] + " ";
					}
					results[k].printToLog(line);
				}
				results[k].printToLogln("");
			}
			results[k].close();
		}
		return this.generateGPSript(directory + File.separator + subFolder, commType);
	}

	public ArrayList<String> generateGPSript(String absolutePath, String commType) {
		ArrayList<String> filesPath = new ArrayList<String>();
		String legend = "right"; // left
		String ss = "";
		
		ss += "# file: myScript.gp displays data keys.dat form of a plot or histogram. \n";
		ss += "# It is possible to set the standard output window X \n";
		ss += "# epstopdf for converting .eps to .pdf \n";
		ss += "set terminal postscript eps enhanced color \"Italic, 16\" #\"Times, 8\" # size 12cm, 8cm \n";
		ss += "set size .8, .75 \n";
		ss += "set size ratio .667 \n";
		ss += "set tmarg 0 \n";
		ss += "set bmarg 2.5 \n";
		ss += "set lmarg 10 \n";
		ss += "set rmarg 2 \n";
		ss += "#set lmargin at screen 0.13, bmargin 0.25 rmargin 0.75 tmargin 0.75 \n";
		ss += "set autoscale \n";
		ss += "set key nobox top "+legend+" spacing 2 Left reverse samplen 4 enhanced autotitles columnhead #box \n";
		ss += "set style data linespoints \n";
		ss += "set pointsize 1.6 \n";
		ss += "set format y \"%1.1l{/Symbol \\327}10^{%L}\" \n";
		ss += "#set format y \"%1.1e\" # \"% g\" # \"%1.1t{/Symbol \\\327}10^{%T}\" \n";
		ss += "#set format y \"%2.0s{/Symbol \\327}10^{%S}\" # \"%1.1e\" \n";
		ss += "#set format x \"%.2f\" \n";
		ss += "#set grid \n";
		ss += "#set border 7 \n";
		ss += "\n";
		ss += "set style line 1 lt 5 lw 2 pt 10 lc rgb \"blue\" \n";
		ss += "set style line 2 lt 1 lw 2 pt 1 lc rgb \"red\" \n";
		ss += "set style line 3 lt 8 lw 2 pt 6 lc rgb \"magenta\" #purple #violet \n";
		ss += "set style line 4 lt 2 lw 2 pt 4 lc rgb \"black\" \n";
		ss += "set style line 5 lt 3 lw 2 pt 3 lc rgb \"coral\" \n";
		ss += "set style line 6 lt 9 lw 4 pt 5 lc rgb \"forest-green\" \n";
		ss += "set style line 7 lt 4 lw 4 pt 4 ps 1 lc rgb \"brown\" \n";
		ss += "set style line 8 lt 6 lw 4 pt 7 ps 1 lc rgb \"dark-turquoise\"  \n";
		ss += "set style line 9 lt 7 lw 4 pt 8 ps 1 lc rgb \"green\" \n";
		ss += "set style line 10 lt 0 lw 4 pt 9 ps 1 lc rgb \"dark-magenta\" \n";
		ss += "set style line 11 lt 1 lw 4 pt 12 ps 1 lc rgb \"cyan\" \n";
		ss += "set style line 12 lt 2 lw 4 pt 2 ps 1 lc rgb \"yellow\" \n";
		ss += "set style line 13 lt 3 lw 4 pt 3 ps 1 lc rgb \"coral\" \n";
		ss += "set xlabel \"" + this.fixKeys + "\" font \"Times Italic, 20\" \n";
		boolean solvable = false;
		for (String key : this.keys) {
			filesPath.add(absolutePath + File.separator + key + ".eps");
			if(key.equals(Commands.SOLVABLE)){
				solvable = true;
				continue;
			}
			ss += "#####################################################################\n";
			if(key.equals(Commands.NCCCs)){
				if (!commType.equals(Commands.FAST_COM)){
					ss += "set output \"e" + key + ".eps\"" + "\n";
					ss += "set ylabel \"#encccs\" font \"Times, 20\" \n";
				} else {
					ss += "set output \"" + key + ".eps\"" + "\n";
					ss += "set ylabel \"#ncccs\" font \"Times, 20\" \n";
				}
			} else if (key.equals(Commands.MSGs)){
				ss += "set output \"" + key + ".eps\"" + "\n";
				ss += "set ylabel \"#msg\" font \"Times, 20\" \n";
			} else {
				ss += "set output \"" + key + ".eps\"" + "\n";
				ss += "set ylabel \"" + key + "\" font \"Times, 20\" \n";
			}
			int length = this.algorithms.length;
			ss += "plot \'" + key + ".dat\' u 1:2 ls "+length;// fs pattern " + 1;
			for (int a = 1 ; a < length ; a++) {
				ss += ", \'\' u 1:" + (a + 2)+" ls "+(length-a); // + " w boxes fs pattern " + (a + 1);
			}
//			ss += "plot \'" + key + ".dat\' u 1:2 ls 1";// fs pattern " + 1;
//			for (int a = 1 ; a < this.algorithms.length ; a++) {
//				ss += ", \'\' u 1:" + (a + 2)+" ls "+(a+1); // + " w boxes fs pattern " + (a + 1);
//			}
			ss += "\npause 0;\n";
		}
		if(solvable){
			ss += "#####################################################################\n";
			ss += "set tmarg 1 \n";
			ss += "set bmarg 5 \n";
			ss += "set yrange [0:101] \n"; 
			ss += "set format y \"%2.0f\" \n";
			ss += "set output \"" + Commands.SOLVABLE + ".eps\"" + "\n";
			ss += "set ylabel \"" + Commands.SOLVABLE + "\" font \"Times, 20\" \n";
//			ss += "set style fill solid 1.0 noborder\n";
//			ss += "set border front linetype -1 linewidth 1.0\n";
			ss += "set style fill solid 1.0 border lt -1 \n";
			ss += "set key default\n";
			ss += "set key bmargin center horizontal Left reverse enhanced autotitles columnhead nobox\n";
			ss += "set style data histograms\n";
			ss += "set style histogram cluster gap "+(this.algorithms.length-1)+"\n";
			ss += "set grid ytics\n";
			ss += "set datafile missing '-'\n";
			ss += "set boxwidth 0.9 relative \n";
//			ss += "plot \'" + Commands.SOLVABLE + ".dat\' u 2:xtic(1)";// fs pattern " + 1;
//			for (int a = 1 ; a < this.algorithms.length ; a++) {
//				ss += ", \'\' u " + (a + 2);// + " w boxes fs pattern " + (a + 1);
//			}
			ss += "plot for [COL=2:"+(this.algorithms.length+1)+":1] \'" + Commands.SOLVABLE + ".dat\' using COL";
			ss += "\npause 0;\n";
		}
		Logger script = new Logger(absolutePath, scriptName);
		script.printToLogln(ss);
		script.close();
		return filesPath;
	}
	
	public void plot(String path, ArrayList<String> filesPath) {
		path = (new File(path)).getAbsolutePath();
		Logger script = new Logger(path, "plot.sh");
//		String ss = "";
//		ss += "#!/bin/bash\n";
//		ss += "for p1 in \'ls\'\n";
//		ss += "do\n";
//		ss += "		if [ -d $p1 ]; then\n";
//		ss += "		#echo \"$p1 \";\n";
//		ss += "		cd $p1;\n";
//		ss += "		chmod +x ./plot.sh;\n";
//		ss += "		"+gnuplot +" myScript.gp\n";
//		ss += "		cd ..\n";
//		ss += "		fi\n";
//		ss += "done\n";
//		ss += "exit 0\n\n";

		String s =
				"#!/bin/sh \n" + 
				"case $# in \n" + 
				"0) \n" + 
				"   path=$(pwd $0);\n" +
				"   cd $path;\n" +
				"   "+gnuplot+" myScript.gp;\n" +
				"   break; \n" + 
				"   ;; \n" +
				"1) \n" + 
				"   "+gnuplot+" $1; \n" + 
				"   break; \n" + 
				"   ;; \n" + 
				"2) \n" + 
				"   if [ ! -d $1 ] \n" + 
				"   then \n" + 
				"       echo \"$1 must be a directory\"; \n" + 
				"       exit 2; \n" + 
				"   else \n" + 
				"       cd $1; \n" + 
				"       if [ ! -f $2 ] \n" + 
				"       then \n" + 
				"           echo \"$2 must be a gnuplot script\"; \n" + 
				"  	        exit 3; \n" + 
				"       else \n" + 
				"           "+gnuplot+" $2; \n" + 
				"       fi \n" + 
				"   fi \n" + 
				"   ;; \n" + 
				"esac \n";
		for(String key : keys){
			s += epstopdf+" ./"+key+".eps;\n";
		}
		s += "echo \"list of generated files is:\" \n" ;
		s += "ls *.eps; \n";
		s += "ls *.pdf; \n";
		s += "exit 0\n";
		script.printToLog(s);
		script.close();
		String cmd1 = "chmod +x " + path + "/plot.sh";
		try {
			Runtime r = Runtime.getRuntime();
			Process p = r.exec(cmd1);
			p.waitFor();
			System.out.println("run : " + cmd1 + " successfully !!");
		} catch (Exception e) {
			System.err.println("Execution error :\n" + e.toString()+"\n");
		}
		String cmd2 = "" + path + "/plot.sh";
		try {
			Runtime r = Runtime.getRuntime();
			Process p = r.exec(cmd2, null, new File(path));
			p.waitFor();
			System.out.println("run : " + cmd2 + " successfully !!");
		} catch (Exception e) {
			System.err.println("Execution error : \n" + e.toString()+"\n");
		}
	}

	public void plotall(String path) {
		Logger script = new Logger(path, "plotall.sh");
		String ss = "";
		ss += "#!/bin/bash\n";
		ss += "for p1 in \'ls\'\n";
		ss += "do\n";
		ss += "		if [ -d $p1 ]; then\n";
		ss += "		#echo \"$p1 \";\n";
		ss += "		cd $p1;\n";
		ss += "		chmod +x ./plot.sh;\n";
		ss += "		"+gnuplot+" myScript.gp\n";
		ss += "		cd ..\n";
		ss += "		fi\n";
		ss += "done\n";
		ss += "exit 0\n\n";
		script.printToLog(ss);
		script.close();
		String cmd1 = "chmod +x " + path + "/plotall.sh";
		try {
			Runtime r = Runtime.getRuntime();
			Process p = r.exec(cmd1);
			p.waitFor();
		} catch (Exception e) {
			System.err.println("Execution error " + cmd1 + e.toString());
		}
		String cmd = "sh " + path + "/plotall.sh " + path + " " + this.scriptName;
		try {
			Runtime r = Runtime.getRuntime();
			Process p = r.exec(cmd);
			p.waitFor();
		} catch (Exception e) {
			System.err.println("Execution error " + cmd + e.toString());
		}
	}

	public void printAllResults(String directory, String header) {
		Logger statistics = new Logger(directory, "commonStatistics.csv"); // common statistics
		Logger allStatistics = new Logger(directory, "allStatistics.csv"); // common statistics
		statistics.printToLogln("#" + header + "\n");
		allStatistics.printToLogln("#" + header + "\n");
//		Data[] data = new Data[this.algorithms.length];
//		for (int p = 0 ; p < this.algorithms.length ; p++) {
//			String fileName = directory + File.separator + this.algorithms[p] + "-" + this.orderings[p] + ".csv";
//			data[p] = Data.fromFile(fileName, this.nbHeaderLine);
//		}
		String[] keys = data[0].getKeys();
		String[] allKeys = data[0].getKeys();
		for (int p = 1 ; p < this.algorithms.length ; p++) {
			keys = Tools.intersection(keys, data[p].getKeys());
			allKeys = Tools.union(allKeys, data[p].getKeys());
		}
		int col = 0;
		String[] KEYS = new String[keys.length - 1]; // -1 fixKey
		for (int k = 0 ; k < keys.length ; k++) {
			if (!keys[k].equals(fixKeys)) {
				KEYS[col] = keys[k];
				col++;
			}
		}
		String algosLine = fixKeys + ", ";
		String keysLine = ",";
		for (int k = 0 ; k < KEYS.length ; k++) {
			keysLine += KEYS[k] + ", ";
			for (int p = 0 ; p < this.algorithms.length - 1 ; p++) {
				keysLine += ",";
				// algosLine += this.algorithms[p] + "-" + this.orderings[p] + ", ";
				algosLine += this.algorithms[p] + ",";
			}
			// algosLine += this.algorithms[this.algorithms.length - 1] + "-" + this.orderings[this.algorithms.length - 1] + ",";
			algosLine += this.algorithms[this.algorithms.length - 1] + ",";
		}
		statistics.printToLogln(keysLine);
		statistics.printToLogln(algosLine);
		for (int l = 0 ; l < data[0].getNbLine() ; l++) {
			col = data[0].getKey(fixKeys);
			String line = " " + data[0].getElement(l)[col] + ",";
			for (int k = 0 ; k < KEYS.length ; k++) {
				for (int p = 0 ; p < this.algorithms.length ; p++) {
					col = data[p].getKey(KEYS[k]);
					line += data[p].getElement(l)[col] + ",";
				}
			}
			statistics.printToLogln(line);
		}
		statistics.close();
		keysLine = " ,";
		algosLine = fixKeys + ",";
		for (String key : allKeys) {
			if (!fixKeys.equals(key)) {
				keysLine += key + "";
				for (String protocol : this.algorithms) {
					keysLine += ",";
					algosLine += protocol + ",";
				}
			}
		}
		allStatistics.printToLogln(keysLine);
		allStatistics.printToLogln(algosLine);
		for (int l = 0 ; l < data[0].getNbLine() ; l++) {
			col = data[0].getKey(fixKeys);
			String line = " " + data[0].getElement(l)[col] + ",";
			for (String key : allKeys) {
				if (!fixKeys.equals(key)) {
					for (int p = 0 ; p < this.algorithms.length ; p++) {
						col = data[p].getKey(key);
						if (col != -1) {
							line += data[p].getElement(l)[col] + ",";
						} else {
							line += "0,";
						}
					}
				}
			}
			allStatistics.printToLogln(line);
		}
		allStatistics.close();
	}

	public XYSeries getSeries(String algoName, String key) {
		XYSeries serie = new XYSeries(algoName, true, false);
		int i = 0;
		for ( ; i < this.algorithms.length ; i++) {
			if (algoName.equals(this.algorithms[i])) {
				break;
			}
		}
		double[] keys = this.data[i].getColumn(this.fixKeys);
		double[] columns = this.data[i].getColumn(key);
		for (int j = 0 ; j < keys.length ; j++) {
			serie.add(keys[j], columns[j]);
		}
		return serie;
	}

	public String getFixKey() {
		return this.fixKeys;
	}

	public ArrayList<String> getKeys() {
		ArrayList<String> keys = new ArrayList<String>();
		for (int i = 0 ; i < this.data.length ; i++) {
			for (String key : this.data[i].getKeys()) {
				if (!keys.contains(key)) {
					keys.add(key);
				}
			}
		}
		keys.remove(this.fixKeys);
		// for(String key : this.fixKeys){
		// keys.remove(key);
		// }
		return keys;
	}

	public double[] getArray(String algoName, String key) {
		int i = 0;
		for ( ; i < this.algorithms.length ; i++) {
			if (algoName.equals(this.algorithms[i])) {
				break;
			}
		}
		return this.data[i].getColumn(key);
	}

	public double[] getFixKeyValues(String algoName) {
		int i = 0;
		for ( ; i < this.algorithms.length ; i++) {
			if (algoName.equals(this.algorithms[i])) {
				break;
			}
		}
		return this.data[i].getColumn(this.fixKeys);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO
		String path = "";
		path = "/Users/wahbi/Documents/workspace/Pc_0.40/";
		String[] names = {
				"ABT-hyb-MaxDegAO.csv", 
				"ABT-MaxDegAO.csv", 
				"AFC-MaxDegAO.csv", 
				"AFC-ng-MaxDegAO.csv", 
				"AFC-tree-TreeAO.csv", 
				};
		String[] files = new String[names.length];
		for (int i = 0 ; i < names.length ; i++) {
			files[i] = path + names[i];
		}
		ResultPrinter printer = new ResultPrinter(files);
		printer.printToGnuPlot(path, "", Commands.FAST_COM);
		printer.plot();
//		String header = "";
//		FileReader file;
//		try {
//			file = new FileReader(files[0]);
//			BufferedReader br = new BufferedReader(file);
//			br.readLine();
//			String line = br.readLine();
//			header += "# " + line+"\n";
//			line = br.readLine();
////			String[] str = line.split(", ");
////			String commType = str[3];
//			header += "# " + line+"\n";
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
//		printer.printAllResults(path, header);
	}
}
