package benchmark.meetingscheduling;

import static gui.Gui.*;
import static util.parser.InstanceTokens.AGENT;
import static util.parser.InstanceTokens.AGENTS;
import static util.parser.InstanceTokens.ARITY;
import static util.parser.InstanceTokens.CONSTRAINT;
import static util.parser.InstanceTokens.CONSTRAINTS;
import static util.parser.InstanceTokens.CONSTRAINT_MODEL;
import static util.parser.InstanceTokens.DESCRIPTION;
import static util.parser.InstanceTokens.DOMAIN;
import static util.parser.InstanceTokens.DOMAINS;
import static util.parser.InstanceTokens.EXPRESSION;
import static util.parser.InstanceTokens.FORMAT;
import static util.parser.InstanceTokens.FUNCTIONAL;
import static util.parser.InstanceTokens.ID;
import static util.parser.InstanceTokens.INSTANCE;
import static util.parser.InstanceTokens.INSTANCES;
import static util.parser.InstanceTokens.MODEL;
import static util.parser.InstanceTokens.NAME;
import static util.parser.InstanceTokens.NB_AGENTS;
import static util.parser.InstanceTokens.NB_CONSTRAINTS;
import static util.parser.InstanceTokens.NB_DOMAINS;
import static util.parser.InstanceTokens.NB_INSTANCES;
import static util.parser.InstanceTokens.NB_PREDICATES;
import static util.parser.InstanceTokens.NB_VALUES;
import static util.parser.InstanceTokens.NB_VARIABLES;
import static util.parser.InstanceTokens.PARAMETERS;
import static util.parser.InstanceTokens.PREDICATE;
import static util.parser.InstanceTokens.PREDICATES;
import static util.parser.InstanceTokens.PRESENTATION;
import static util.parser.InstanceTokens.REFERENCE;
import static util.parser.InstanceTokens.SCOPE;
import static util.parser.InstanceTokens.TKC;
import static util.parser.InstanceTokens.TYPE;
import static util.parser.InstanceTokens.VARIABLE;
import static util.parser.InstanceTokens.VARIABLES;
import static util.parser.InstanceTokens.XDisCSP_1_0;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import dischoco.DisChoco;
import benchmark.ResultPrinter;
import gui.GuiSimulator;
import kernel.order.AgentOrdering;
import simulation.DisProblem;
import simulation.DisSolverSimulator;
import util.Commands;
import util.IntArray;
import util.Logger;
import util.Tools;

/**
 * Uniform Random Meeting Scheduling Problem Generator
 * 
 * 
 * @authors 
 * <a href="mailto:ezzahir@ensa-agadir.ac.ma?subject=DisChoco &cc=wahbi@lirmm.fr &cc=bessiere@lirmm.fr">
 * Redoaune Ezzahir</a> and 
 * <a href="mailto:wahbi@lirmm.fr?subject=DisChoco &cc=ezzahir@ensa-agadir.ac.ma &cc=bessiere@lirmm.fr">
 * Mohamed Wahbi</a>
 *
 */
public class URMeetingScheduling extends GuiSimulator  {
	/**
	 * 
	 */
	private boolean DEBUG = false;
	/**
	 * the geographic places when meetings well take place
	 */

	public final static String fixKeys = NB_MEETINGS_PER_AGENT;

	public static int nbHeaderLine = 4;
	private static int meetPerAgent;
	private static int instance;
	private static int run;
	/** the number of instances to generate for each class */
	private int nbInstances = 25;
	/** the ranges of number of agents */
	private int minNbAgents, maxNbAgents, stepNbAgents;
	/** the ranges of number of meetings */
	private int minNbMeetings, maxNbMeetings, stepNbMeetings;
	/** the ranges of number of meeting per agent */
	private int minNbMeetingsPerAgent, maxNbMeetingsPerAgent, stepNbMeetingsPerAgent = 0;
	/** The geography that defines cities of meetings locations and distances */
	private Geography geography;
	/** for each meeting there are a set of agents that attend it (attendances) */
	private short[][] attendances; 
	/** to store the edges of the meeting graph */
	private short[][] meetingsAdjacences;
	/** for each agent there are a slots */
	private Slot[] slots;
	/**  for each meeting there is a place */
	private Place[] places;
	private short minParticipantsPerMeet = 2;
	private int meetingsDuration = 1;
	private File directory = new File(Commands.MSP);
	/**  numbers of days to put in the slots */
	private int nbDays = 5;
	/**  numbers of hours in the day to put in the slots */
	private int nbHours = 10;
	/**  percentage of disponibility over <i>numbers of hours</i> <code>x</code> <i>numbers of days</i> */
	private int disponibility = 100;
	private int x_axis = 10;
	private int y_axis = 10;

	public URMeetingScheduling(File directory, int nbInstances, int nbMeetings, int nbAgents, int nbMeetingsPerAgent, Geography geography,
			Slot[] slots, int meetingsDuration) {
		this.directory = directory;
		this.nbInstances = nbInstances;
		this.geography = geography;
		this.slots = slots;
		this.meetingsDuration = meetingsDuration;
		if (!this.directory.isDirectory()) {
			this.directory.mkdir();
		}
	}

	public URMeetingScheduling(File directory, int nbInstances, int nbMeetings, int nbAgents, int nbMeetingsPerAgent, int meetingsDuration,
			int disponibility, int x_axis, int y_axis) {
		this.directory = directory;
		this.nbInstances = nbInstances;
		this.slots = new Slot[nbAgents];
		this.meetingsDuration = meetingsDuration;
		this.slots = new Slot[nbAgents];
		for (int a = 0 ; a < nbAgents ; a++) {
			slots[a] = Slot.generateWeek(this.nbDays, this.nbHours, disponibility);
		}
		this.x_axis = x_axis;
		this.y_axis = y_axis;
		this.geography = Geography.generateGrid(x_axis, y_axis);
	}

	public URMeetingScheduling(File directory, int nbInstances, int meetingsDuration) {
		this.directory = directory;
		if (!this.directory.isDirectory()) {
			this.directory.mkdir();
		}
		this.nbInstances = nbInstances;
		this.meetingsDuration = meetingsDuration;
	}

	/**
	 * @param filePath
	 */
	public URMeetingScheduling() {
		super();
	}

	public void setMeetingsProperties(int minNbMeetings, int maxNbMeetings, int step) {
		this.minNbMeetings = minNbMeetings;
		this.maxNbMeetings = maxNbMeetings;
		this.stepNbMeetings = step;
	}

	public void setAgentsProperties(int minNbAgents, int maxNbAgents, int step) {
		this.minNbAgents = minNbAgents;
		this.maxNbAgents = maxNbAgents;
		this.stepNbAgents = step;
	}

	public void setMeetingsPerAgentsProperties(int min, int max, int step) {
		this.minNbMeetingsPerAgent = min;
		this.maxNbMeetingsPerAgent = max;
		this.stepNbMeetingsPerAgent = step;
	}

	public void setGeographyProperties(int x_axis, int y_axis) {
		this.x_axis = x_axis;
		this.y_axis = y_axis;
		this.geography = Geography.generateGrid(x_axis, y_axis);
	}

	public void setCalendarProperties(int nbDays, int nbHours, int disponibility) {
		this.nbDays = nbDays;
		this.nbHours = nbHours;
		this.disponibility = disponibility;
	}

	public void generateCalendares(int nbAgents) {
		this.slots = new Slot[nbAgents];
		for (int a = 0 ; a < nbAgents ; a++) {
			slots[a] = Slot.generateWeek(this.nbDays, this.nbHours, this.disponibility);
		}
	}

	private void initializer(int nbMeetings, int nbAgents) {
		this.attendances = new short[nbMeetings][nbAgents];
		for (int i = 0 ; i < this.attendances.length ; i++) {
			for (int j = 0 ; j < this.attendances[i].length ; j++) {
				this.attendances[i][j] = 0;
			}
		}
		this.meetingsAdjacences = new short[nbMeetings][nbMeetings];
		for (int i = 0 ; i < meetingsAdjacences.length ; i++) {
			for (int j = 0 ; j < meetingsAdjacences[i].length ; j++) {
				meetingsAdjacences[i][j] = 0;
			}
		}
	}

	/**
	 * select for each meeting a place from the geography
	 */
	private void selectPlacesForMeetings(int nbMeetings) {
		this.places = new Place[nbMeetings];
		for (int p = 0 ; p < this.places.length ; p++) {
			this.places[p] = this.geography.getRandomPlace();
			if (this.DEBUG) {
				System.out.println("M" + (p + 1) + " will be hold at " + this.places[p]);
			}
		}
		if (this.DEBUG) {
			System.out.print("----------------------");
			for (int p1 = 0 ; p1 < this.places.length ; p1++) {
				System.out.print("M" + (p1 + 1) + " ");
			}
			System.out.println("");
			for (int p1 = 0 ; p1 < this.places.length ; p1++) {
				System.out.print("M" + (p1 + 1) + " ");
				for (int p2 = 0 ; p2 < this.places.length ; p2++) {
					System.out.print("" + this.getDistance(p1, p2) + "  ");
				}
				System.out.println("");
			}
		}
	}

	/**
	 * select randomly a clique of k meetings | not all of edges of the clique are already in the network.
	 */
	private boolean generateMeetings(int nbMeetings, int nbAgents, int nbMeetingsPerAgent) {
		this.initializer(nbMeetings, nbAgents);
		Random random = new Random();
		short[] nbParticipantsPerMeet = new short[nbMeetings];
		for (int a = 0 ; a < nbParticipantsPerMeet.length ; a++) {
			nbParticipantsPerMeet[a] = 0;
		}
		if (this.DEBUG) {
			System.out.println("generate meetings:");
		}
		ArrayList<int[]> combinations = getAllCombinations(nbMeetings, nbMeetingsPerAgent);
		int[] index = new int[combinations.size()];
		for (int i = 0 ; i < index.length ; i++) {
			index[i] = i;
		}
		for (int agId = 0 ; agId < nbAgents ; agId++) {
			int selected = random.nextInt(index.length - agId);
			index[selected] = index[index.length - agId - 1];
			int[] meetings = combinations.get(selected);
			for (int m_i = 0 ; m_i < meetings.length ; m_i++) {
				this.attendances[meetings[m_i]][agId] = 1;
				nbParticipantsPerMeet[meetings[m_i]]++;
				for (int m_j = m_i + 1 ; m_j < meetings.length ; m_j++) {
					this.meetingsAdjacences[meetings[m_i]][meetings[m_j]] = 1;
					this.meetingsAdjacences[meetings[m_j]][meetings[m_i]] = 1;
				}
			}
		}
		if(DEBUG) System.err.println(toString(nbAgents));
		for (int a = 0 ; a < nbParticipantsPerMeet.length ; a++) {
			if (nbParticipantsPerMeet[a] < this.minParticipantsPerMeet) {
				// TODO
				//System.exit(0);
				return false;
			}
		}
		if (this.DEBUG) {
			this.println(nbAgents);
		}
		return true;
	}

	protected void println(int nbAgents) {
		System.out.println(this.toString());
		for (int a = 0 ; a < nbAgents ; a++) {
			int[] domain = this.slots[a].toIntDomain();
			String slot = "";
			for (int d = 0 ; d < domain.length ; d++) {
				slot += domain[d] + " ";
			}
			System.out.println("A" + a + " : " + slot);
		}
	}

	public String toString(int nbAgents) {
		String ans = "------------------------------------------------------\n";
		ans += "Meeting\t | Location | Attending agents\n";
		for (int m = 0 ; m < this.attendances.length ; m++) {
			ans += "M" + (m + 1) + "  \t |  " + this.places[m] + "   | ";
			int[] agents = this.getParticipantsOn(m);
			for (int a = 0 ; a < agents.length ; a++) {
				ans += "A" + (agents[a] + 1);
				if (a < agents.length - 1) {
					ans += ", ";
				}
			}
			ans += "\n";
		}
		ans += "------------------------------------------------------\n";
		ans += "Agents   | Meetings\n";
		for (int a = 0 ; a < nbAgents ; a++) {
			ans += "A" + (a + 1) + "  \t |  ";
			int[] meetings = this.getMeetingsOf(a);
			for (int m = 0 ; m < meetings.length ; m++) {
				ans += "M" + (meetings[m] + 1);
				if (m < meetings.length - 1) {
					ans += ", ";
				}
			}
			ans += "\n";
		}
		ans += "------------------------------------------------------\n";
		return ans;
	}

	private Element generateInstance(int nbMeetings, int nbAgents, int nbMeetingsPerAgent) {
//		if(nbAgents*nbMeetingsPerAgent/2 < nbMeetings){
//			throw new Error("\n" +
//					"Impossible to generate this problem:\n" + "not enough agent!!\n" + 
//					NB_MEETINGS + " = "+ nbMeetings + "\n" + NB_AGENTS + " = " + nbAgents + "\n" + 
//					NB_MEETINGS_PER_AGENT + " = " + nbMeetingsPerAgent);
//		}
		if(Tools.C(nbAgents, minParticipantsPerMeet) < nbMeetings){
			throw new Error("\n" +
					"Impossible to generate this problem:\n" + "not enough agent!!\n" + 
					NB_MEETINGS + " = "+ nbMeetings + "\n" + NB_AGENTS + " = " + nbAgents + "\n" + 
					NB_MEETINGS_PER_AGENT + " = " + nbMeetingsPerAgent);
		}
		if(Tools.C(nbMeetings, nbMeetingsPerAgent) < nbAgents){
			throw new Error("\n" +
					"Impossible to generate this problem:\n" + "not enough agent!!\n" + 
					NB_MEETINGS + " = "+ nbMeetings + "\n" + NB_AGENTS + " = " + nbAgents + "\n" + 
					NB_MEETINGS_PER_AGENT + " = " + nbMeetingsPerAgent);
		}
		// generate for each agent a calendar;
		this.generateCalendares(nbAgents);
		// select for each meeting a place from the geography
		this.selectPlacesForMeetings(nbMeetings);
		// select randomly a clique of k meetings | not all of edges
		// of the clique are already in the network.
		int i = 0;
		boolean uniform = false;
		while (!uniform) {
			i++;
			uniform = this.generateMeetings(nbMeetings, nbAgents, nbMeetingsPerAgent);
			if (i > 1000000) { 
				System.err.println(toString(nbAgents));
				throw new Error("\n" +
						"You can not generate a uniform Meeting scheduling Problem with\n" + 
						NB_MEETINGS + " = "+ nbMeetings + "\n" + NB_AGENTS + " = " + nbAgents + "\n" + 
						NB_MEETINGS_PER_AGENT + " = " + nbMeetingsPerAgent);
			}
		}
		return this.getInstance(nbMeetings, nbAgents, nbMeetingsPerAgent);
	}

	// public void generateOne(){
	// if(nbAgents*this.nbMeetingsPerAgent<this.minParticipantsPerMeet*this.nbMeetings){
	// System.err.println("Impossible to generate this problem:\n"+
	// "not enough agent!!"); // enough
	// return;
	// }
	// //select for each meeting a place from the geography
	// this.selectPlacesForMeetings();
	// //select randomly a clique of k meetings | not all of edges
	// //of the clique are already in the network.
	// boolean uniform = false;
	// while(!uniform){
	// uniform = this.generateMeetings();
	// }
	// Element instance = this.getInstance();
	// Document document = new Document(instance);
	// this.saveInDirectory(document, Commands.MSP+""+".xml");
	// if(this.DEBUG){
	// this.println();
	// }
	// }

	/**
	 * @param subDirectory
	 * @param fileName
	 */
	void saveInDirectory(Document document, String fileName) {
		try {
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			out.output(document, new FileOutputStream(this.directory.getPath() + File.separator + fileName));
		} catch (Exception e) {}
	}

	/**
	 * @param subDirectory
	 * @param fileName
	 */
	void save(String directory, Document document, String file) {
		try {
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			out.output(document, new FileOutputStream(directory + File.separator + file));
			Process p = Runtime.getRuntime().exec("bzip2 " + directory + File.separator + file);
			p.waitFor();
			p.exitValue();
			p.destroy();
		} catch (Exception e) {}
	}

	public Element getInstance(int nbMeetings, int nbAgents, int nbMeetingsPerAgent) {
		Element instance = new Element(INSTANCE);
		instance.addContent(this.getPresentation());
		instance.addContent(this.getAgents(nbAgents));
		instance.addContent(this.getDomaines(nbAgents));
		instance.addContent(this.getVariables(nbMeetings, nbAgents, nbMeetingsPerAgent));
		instance.addContent(getPredicates());
		instance.addContent(this.getConstraints(nbAgents));
		instance.addContent(this.getGuiPresentation(nbMeetings, nbAgents, nbMeetingsPerAgent));
		return instance;
	}

	private Element getPresentation() {
		Element presentation = new Element(PRESENTATION);
		presentation.setAttribute(NAME, Commands.MeetingScheduling);
		presentation.setAttribute(TYPE, Commands.DMSP);
		presentation.setAttribute(MODEL, Commands.COMPLEX);
		presentation.setAttribute(CONSTRAINT_MODEL, TKC);
		presentation.setAttribute(FORMAT, XDisCSP_1_0);
		return presentation;
	}

	private Element getGuiPresentation(int nbMeetings, int nbAgents, int nbMeetingsPerAgent) {
		Element presentation = new Element(GuiPresentation);
		presentation.setAttribute(NAME, Commands.MeetingScheduling);
		presentation.setAttribute(NB_MEETINGS, nbMeetings+"");
		presentation.setAttribute(NB_AGENTS, nbAgents+"");
		presentation.setAttribute(NB_MEETINGS_PER_AGENT, nbMeetingsPerAgent+"");
		presentation.setAttribute(DISPONIBILITY, this.disponibility+"");
		presentation.setAttribute(NB_DAYS, this.nbDays+"");
		presentation.setAttribute(NB_HOURS, this.nbHours+"");
		presentation.setAttribute(X_axis, this.x_axis+"");
		presentation.setAttribute(Y_axis, this.y_axis+"");
		return presentation;
	}

	private Element getAgents(int nbAgents) {
		Element agents = new Element(AGENTS);
		agents.setAttribute(NB_AGENTS, "" + nbAgents);
		for (int id = 1 ; id <= nbAgents ; id++) {
			Element agent = new Element(AGENT);
			agent.setAttribute(NAME, "A" + id);
			agent.setAttribute(ID, "" + id);
			agent.setAttribute(DESCRIPTION, AGENT + id);
			agents.addContent(agent);
		}
		return agents;
	}

	private Element getDomaines(int nbAgents) {
		Element domains = new Element(DOMAINS);
		domains.setAttribute(NB_DOMAINS, "" + nbAgents);
		for (int id = 0 ; id < nbAgents ; id++) {
			Element domain = new Element(DOMAIN);
			domain.setAttribute(NAME, "D" + (id + 1));
			int[] values = this.slots[id].toIntDomain();
			domain.setAttribute(NB_VALUES, "" + values.length);
			domain.setText("" + Tools.arrayToString(values));
			domains.addContent(domain);
		}
		return domains;
	}

	private Element getVariables(int nbMeetings, int nbAgents, int nbMeetingsPerAgent) {
		Element variables = new Element(VARIABLES);
		variables.setAttribute(NB_VARIABLES, "" + nbAgents * nbMeetingsPerAgent);
		for (int a = 0 ; a < nbAgents ; a++) {
			for (int m = 0 ; m < nbMeetings ; m++) {
				if (this.attendances[m][a] == 1) {
					Element variable = new Element(VARIABLE);
					variable.setAttribute(AGENT, "A" + (a + 1));
					variable.setAttribute(NAME, "M" + (a + 1) + "." + (m + 1));
					// variable.setAttribute(ID, (a+1)+"."+(m+1));
					variable.setAttribute(ID, "" + (m + 1));
					variable.setAttribute(DOMAIN, "D" + (a + 1));
					variable.setAttribute(DESCRIPTION, "M_" + (m + 1));
					variables.addContent(variable);
				}
			}
		}
		return variables;
	}

	/**
	 * 
	 */
	private Element getPredicates() {
		Element predicates = new Element(PREDICATES);
		predicates.setAttribute(NB_PREDICATES, "" + 2);
		// ///////////////////////////////////////////////////////////////////////
		Element nePredicate = new Element(PREDICATE);
		nePredicate.setAttribute(NAME, "P1");
		Element neParameters = new Element(PARAMETERS);
		neParameters.setText(" int Mi int Mj int DD ");
		nePredicate.addContent(neParameters);
		Element neExpression = new Element(EXPRESSION);
		Element neFunctional = new Element(FUNCTIONAL);
		neFunctional.setText(" ge(abs(sub(Mi,Mj)), DD) "); // it is not
		// necessary to say
		// Mi!=Mj
		neExpression.addContent(neFunctional);
		nePredicate.addContent(neExpression);
		predicates.addContent(nePredicate);
		// ///////////////////////////////////////////////////////////////////////
		Element eqPredicate = new Element(PREDICATE);
		eqPredicate.setAttribute(NAME, "P0");
		Element eqParameters = new Element(PARAMETERS);
		eqParameters.setText("int Mi int Mj ");
		eqPredicate.addContent(eqParameters);
		Element eqExpression = new Element(EXPRESSION);
		Element eqFunctional = new Element(FUNCTIONAL);
		eqFunctional.setText(" eq(Mi,Mj) ");
		eqExpression.addContent(eqFunctional);
		eqPredicate.addContent(eqExpression);
		predicates.addContent(eqPredicate);
		return predicates;
	}

	/**
	 * 
	 */
	private Element getConstraints(int nbAgents) {
		Element constraints = new Element(CONSTRAINTS);
		int nbConstraints = 0;
		// ///////////////////////////////////////////////////////////////////////////////////
		for (int m = 0 ; m < this.attendances.length ; m++) {
			int[] agents = this.getParticipantsOn(m);
			for (int i = 0 ; i < agents.length - 1 ; i++) {
				for (int j = i + 1 ; j < agents.length ; j++) {
					int a_i = agents[i];
					int a_j = agents[j];
					Element constraint = new Element(CONSTRAINT);
					constraint.setAttribute(NAME, "C" + (nbConstraints));
					constraint.setAttribute(MODEL, Commands.TKC);
					constraint.setAttribute(ARITY, "2");
					// constraint.setAttribute(SCOPE_AGENTS,
					// "A"+(a_i+1)+" A"+(a_j+1));
					constraint.setAttribute(SCOPE, "M" + (a_i + 1) + "." + (m + 1) + " M" + (a_j + 1) + "." + (m + 1));
					constraint.setAttribute(REFERENCE, "P0");
					Element parameters = new Element(PARAMETERS);
					parameters.setText("M" + (a_i + 1) + "." + (m + 1) + " M" + (a_j + 1) + "." + (m + 1) + " ");
					constraint.addContent(parameters);
					constraints.addContent(constraint);
					nbConstraints++;
				}
			}
		}
		// ///////////////////////////////////////////////////////////////////////////////////
		for (int a = 0 ; a < nbAgents ; a++) {
			int[] meetings = this.getMeetingsOf(a);
			for (int i = 0 ; i < meetings.length - 1 ; i++) {
				for (int j = i + 1 ; j < meetings.length ; j++) {
					int m_i = meetings[i];
					int m_j = meetings[j];
					Element constraint = new Element(CONSTRAINT);
					constraint.setAttribute(NAME, "C" + (nbConstraints));
					constraint.setAttribute(MODEL, Commands.TKC);
					constraint.setAttribute(ARITY, "2");
					// constraint.setAttribute(SCOPE_AGENTS,
					// "A"+(a+1)+" A"+(a+1));
					constraint.setAttribute(SCOPE, "M" + (a + 1) + "." + (m_i + 1) + " M" + (a + 1) + "." + (m_j + 1));
					constraint.setAttribute(REFERENCE, "P1");
					Element parameters = new Element(PARAMETERS);
					int dd = this.meetingsDuration + this.getDistance(m_i, m_j);
					parameters.setText("M" + (a + 1) + "." + (m_i + 1) + " M" + (a + 1) + "." + (m_j + 1) + " " + dd);
					constraint.addContent(parameters);
					constraints.addContent(constraint);
					nbConstraints++;
				}
			}
		}
		// ///////////////////////////////////////////////////////////////////////////////////
		constraints.setAttribute(NB_CONSTRAINTS, "" + nbConstraints);
		return constraints;
	}

	private int getDistance(int m_i, int m_j) {
		double distance = this.places[m_i].distance(this.places[m_j]);
		if (((double) distance % 1.0) != 0.0) { return (int) distance + 1; }
		return (int) distance;
	}

	private int[] getMeetingsOf(int a) {
		IntArray meetings = new IntArray();
		for (int m = 0 ; m < this.attendances.length ; m++) {
			if (this.attendances[m][a] == 1) {
				meetings.add(m);
			}
		}
		return meetings.getArray();
	}

	private int[] getParticipantsOn(int m) {
		IntArray agents = new IntArray();
		for (int a = 0 ; a < this.attendances[m].length ; a++) {
			if (this.attendances[m][a] == 1) {
				agents.add(a);
			}
		}
		return agents.getArray();
	}

	/**
	 * 
	 */
	public void generate() {
		Element experiments = new Element(Commands.Experiments);
		Document confdocument = new Document(experiments);
		experiments.setAttribute(Commands.type, Commands.MeetingScheduling);
		Element instances = new Element(INSTANCES);
		experiments.addContent(instances);
		instances.setAttribute(TYPE, Commands.MeetingScheduling);
		instances.setAttribute(MEETINGS_DURATION, "" + this.meetingsDuration);
		instances.setAttribute(NB_DAYS, "" + this.nbDays);
		instances.setAttribute(NB_HOURS, "" + this.nbHours);
		instances.setAttribute(DISPONIBILITY, "" + this.disponibility);
		instances.setAttribute(X_axis, "" + this.x_axis);
		instances.setAttribute(Y_axis, "" + this.y_axis);
		String name = Commands.MSP + "_";
		instances.setAttribute(NAME, name);
		for (int nbMeetings = this.minNbMeetings ; nbMeetings <= this.maxNbMeetings ; nbMeetings += this.stepNbMeetings) {
			for (int nbAgents = this.minNbAgents ; nbAgents <= this.maxNbAgents ; nbAgents += this.stepNbAgents) {
				String subDensity = "M-" + nbMeetings + "_A-" + nbAgents;
				File folder = new File(this.directory + File.separator + subDensity);
				if (!folder.exists()) { folder.mkdir(); }
				Element dir = new Element(DIRECTORY);
				dir.setAttribute(NAME, subDensity);
				dir.setAttribute(NB_MEETINGS, "" + nbMeetings);
				dir.setAttribute(NB_AGENTS, "" + nbAgents);
				dir.setAttribute(MIN_MEETING_PER_AGENTS, "" + this.minNbMeetingsPerAgent);
				dir.setAttribute(MAX_MEETING_PER_AGENTS, "" + this.maxNbMeetingsPerAgent);
				dir.setAttribute(STEP_MEETING_PER_AGENTS, "" + this.stepNbMeetingsPerAgent);
				// System.out.println(" ===== "+subDensity);
				for (int nbMperA = this.minNbMeetingsPerAgent ; nbMperA <= this.maxNbMeetingsPerAgent ; nbMperA += this.stepNbMeetingsPerAgent) {
					// this.nbMeetingsPerAgent = nbMperA;
					// System.out.println(" M_A = "+nbMperA);
					File subFolder = new File(this.directory + File.separator + subDensity + File.separator + nbMperA);
					if (!subFolder.exists()) {
						subFolder.mkdir();
					}
					for (int i = 0 ; i < this.nbInstances ; i++) {
						// System.out.println(" instance = "+i);
						Document document = new Document(this.generateInstance(nbMeetings, nbAgents, nbMperA));
						this.save(subFolder.getPath(), document, name + (i + 1) + ".xml");
					}
				}
				dir.setAttribute(NB_INSTANCES, "" + this.nbInstances);
				instances.addContent(dir);
			}
		}
		this.saveInDirectory(confdocument, Commands.MeetingScheduling+"_"+nbInstances+ ".xml");
	}

	@SuppressWarnings("unchecked")
	public static boolean lunchXP(File file) {
		ArrayList<String> keys = null;
		String filePath = file.getParentFile().getAbsolutePath();
		// //////////////////////////////////////////////////////////////////////////////////////////////////////////
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(file);
		} catch (Exception e) {}
		Element xp = document.getRootElement();
		Element instances = xp.getChild(INSTANCES);
		List<Element> communications = xp.getChild(Commands.Communications).getChildren();
		List<Element> algorithms = xp.getChild(Commands.Algorithms).getChildren();
		String[] protocols = new String[algorithms.size()];
		String[] orderings = new String[algorithms.size()];
		int a = 0;
		Iterator<Element> it = algorithms.iterator();
		while (it.hasNext()) {
			Element algorithm = it.next();
			protocols[a] = algorithm.getText();
			orderings[a] = algorithm.getAttributeValue(Commands.AO);
			a++;
		}
		int meetingDuration = new Integer(instances.getAttributeValue(MEETINGS_DURATION));
		int nbDays = new Integer(instances.getAttributeValue(NB_DAYS));
		int nbHours = new Integer(instances.getAttributeValue(NB_HOURS));
		int disponibility = new Integer(instances.getAttributeValue(DISPONIBILITY));
		int x_axis = new Integer(instances.getAttributeValue(X_axis));
		int y_axis = new Integer(instances.getAttributeValue(Y_axis));
		String name = instances.getAttribute(NAME).getValue();
		it = communications.iterator();
		while (it.hasNext()) {
			// /////////////////////////////////////////////////////////////////
			Element comm = it.next();
			String commType = comm.getAttributeValue(TYPE);
			String channelType = comm.getAttributeValue(Commands.channelType);
			int minURC = new Integer(comm.getAttributeValue(Commands.MinURC));
			int maxURC = new Integer(comm.getAttributeValue(Commands.MaxURC));
			int nbRun = new Integer(comm.getAttributeValue(Commands.NbRun));
			long maxNCCCs = new Long(comm.getAttributeValue(Commands.MaxNCCCs));
			// /////////////////////////////////////////////////////////////////
			List<Element> lines = instances.getChildren();
			for (int l = 0 ; l < lines.size() ; l++) {
				Element line = lines.get(l);
				int nbMeetings = new Integer(line.getAttributeValue(NB_MEETINGS));
				int nbAgents = new Integer(line.getAttributeValue(NB_AGENTS));
				int minMeetPerAgent = new Integer(line.getAttributeValue(MIN_MEETING_PER_AGENTS));
				int maxMeetPerAgent = new Integer(line.getAttributeValue(MAX_MEETING_PER_AGENTS));
				int stepMeetPerAgent = new Integer(line.getAttributeValue(STEP_MEETING_PER_AGENTS));
				int nbInstance = new Integer(line.getAttributeValue(NB_INSTANCES));
				String problemsFolder = filePath + File.separator + line.getAttributeValue(NAME);
				String resultsDirectory = filePath + File.separator + Commands.Results + "_" + nbInstance+"_"+nbRun;
				try {
					File f = new File(resultsDirectory);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				resultsDirectory += File.separator + commType + "_" + channelType;
				try {
					File f = new File(resultsDirectory);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				String subFolder = resultsDirectory + File.separator + line.getAttributeValue(NAME);
				try {
					File f = new File(subFolder);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				for (int p = 0 ; p < protocols.length ; p++) { // Protocols
					Logger allResults = new Logger(subFolder, "All_" + protocols[p] + "-" + orderings[p] + ".csv");
					Logger results = new Logger(subFolder, protocols[p] + "-" + orderings[p] + ".csv");
					for (meetPerAgent = minMeetPerAgent ; meetPerAgent <= maxMeetPerAgent ; meetPerAgent += stepMeetPerAgent) {
						HashMap<String, Double> densityStatistics = new HashMap<String, Double>();
						for (instance = 1 ; instance <= nbInstance ; instance++) {
							String problem = problemsFolder + File.separator + meetPerAgent + File.separator + name + instance + ".xml.bz2";
							HashMap<String, Double> instanceStatistics = new HashMap<String, Double>();
							for (run = 1 ; run <= nbRun ; run++) {
								DisProblem disProb = new DisProblem(problem, protocols[p]);
								DisSolverSimulator solver = new DisSolverSimulator(disProb);
								solver.setCutOffNCCCs(maxNCCCs);
								solver.addCommunicationMetric();
								solver.setMessageDelay(minURC, maxURC);
								solver.setCentralizedAO(AgentOrdering.getCentralizedAO(orderings[p]));
								solver.solve();
								if(keys == null ){
									keys = disProb.master.getStatisticsKeys();
								}
								HashMap<String, Double> runStatistics = solver.getStatistics();
								if (instance == 1 && run == 1 && meetPerAgent == minMeetPerAgent) {
									String s = ",";
									String header = Commands.MeetingScheduling + s;
									header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
									header += NB_MEETINGS + s + nbMeetings + s + NB_AGENTS + s + nbAgents + s ;
									header += MEETINGS_DURATION + s + meetingDuration + s + NB_DAYS + s + nbDays + s ;
									header += NB_HOURS + s + nbHours + s + DISPONIBILITY + s + disponibility + s ;
									header += X_axis + s + x_axis + s + Y_axis + s + y_axis + s;
									header += NB_INSTANCES+s+nbInstance + s + Commands.NbRun+s+nbRun+"\n";
									header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s ;
									header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "\n\n";
									header += NB_MEETINGS_PER_AGENT + s ;
									allResults.printToLogln(header + INSTANCE + s + Commands.NbRun + s + Tools.toString(keys));
									results.printToLogln(header + Tools.toString(keys));
								}
								allResults.printToLogln(meetPerAgent + "," + instance + "," + run + "," + Tools.statisticsToString(keys, runStatistics));
								instanceStatistics = Tools.merge(runStatistics, instanceStatistics);
								disProb.setAgents(null);
								System.gc();
							}
							Tools.average(instanceStatistics, nbRun);
							densityStatistics = Tools.merge(instanceStatistics, densityStatistics);
						}
						Tools.average(densityStatistics, nbInstance);
						results.printToLogln(meetPerAgent + "," + Tools.statisticsToString(keys, densityStatistics));
					}
					results.close();
					allResults.close();
					keys = null;
				} //Protocols
				String s = ",";
				ResultPrinter printer = new ResultPrinter(protocols, orderings, nbHeaderLine);
				printer.setFixKeys(fixKeys);
				ArrayList<String> paths = printer.printToGnuPlot(subFolder, "", commType);
				printer.plot(subFolder, paths);

				String header = Commands.MeetingScheduling + s;
				header += NB_MEETINGS + s + nbMeetings + s + NB_AGENTS + s + nbAgents + s ;
				header += MEETINGS_DURATION + s + meetingDuration + s + NB_DAYS + s + nbDays + s ;
				header += NB_HOURS + s + nbHours + s + DISPONIBILITY + s + disponibility + s ;
				header += X_axis + s + x_axis + s + Y_axis + s + y_axis + s;
				header += NB_INSTANCES+s+nbInstance + s + Commands.NbRun+s+nbRun+"\n";
				header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s ;
				header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "";
				printer.printAllResults(subFolder, header);
			}
		}
		return true;
	}

	@SuppressWarnings("unchecked")
	@Override
	public void lunch() {
		ArrayList<String> keys = null;
		String filePath = DisChoco.filePath.getParent();
		// //////////////////////////////////////////////////////////////////////////////////////////////////////////
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(DisChoco.filePath);
		} catch (Exception e) {}
		Element xp = document.getRootElement();
		Element instances = xp.getChild(INSTANCES);
		List<Element> communications = xp.getChild(Commands.Communications).getChildren();
		List<Element> algorithms = xp.getChild(Commands.Algorithms).getChildren();
		String[] protocols = new String[algorithms.size()];
		String[] orderings = new String[algorithms.size()];
		int a = 0;
		Iterator<Element> it = algorithms.iterator();
		while (it.hasNext()) {
			Element algorithm = it.next();
			protocols[a] = algorithm.getText();
			orderings[a] = algorithm.getAttributeValue(Commands.AO);
			a++;
		}
		int meetingDuration = new Integer(instances.getAttributeValue(MEETINGS_DURATION));
		int nbDays = new Integer(instances.getAttributeValue(NB_DAYS));
		int nbHours = new Integer(instances.getAttributeValue(NB_HOURS));
		int disponibility = new Integer(instances.getAttributeValue(DISPONIBILITY));
		int x_axis = new Integer(instances.getAttributeValue(X_axis));
		int y_axis = new Integer(instances.getAttributeValue(Y_axis));
		String name = instances.getAttribute(NAME).getValue();
		it = communications.iterator();
		while (it.hasNext()) {
			// /////////////////////////////////////////////////////////////////
			Element comm = it.next();
			String commType = comm.getAttributeValue(TYPE);
			String channelType = comm.getAttributeValue(Commands.channelType);
			int minURC = new Integer(comm.getAttributeValue(Commands.MinURC));
			int maxURC = new Integer(comm.getAttributeValue(Commands.MaxURC));
			int nbRun = new Integer(comm.getAttributeValue(Commands.NbRun));
			long maxNCCCs = new Long(comm.getAttributeValue(Commands.MaxNCCCs));
			// /////////////////////////////////////////////////////////////////
			List<Element> lines = instances.getChildren();
			for (int l = 0 ; l < lines.size() ; l++) {
				Element line = lines.get(l);
				int nbMeetings = new Integer(line.getAttributeValue(NB_MEETINGS));
				int nbAgents = new Integer(line.getAttributeValue(NB_AGENTS));
				int minMeetPerAgent = new Integer(line.getAttributeValue(MIN_MEETING_PER_AGENTS));
				int maxMeetPerAgent = new Integer(line.getAttributeValue(MAX_MEETING_PER_AGENTS));
				int stepMeetPerAgent = new Integer(line.getAttributeValue(STEP_MEETING_PER_AGENTS));
				int nbInstance = new Integer(line.getAttributeValue(NB_INSTANCES));
				String problemsFolder = filePath + File.separator + line.getAttributeValue(NAME);
				String resultsDirectory = filePath + File.separator + Commands.Results + "_" + nbInstance+"_"+nbRun;
				try {
					File f = new File(resultsDirectory);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				resultsDirectory += File.separator + commType + "_" + channelType;
				try {
					File f = new File(resultsDirectory);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				String subFolder = resultsDirectory + File.separator + line.getAttributeValue(NAME);
				try {
					File f = new File(subFolder);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				for (int p = 0 ; p < protocols.length ; p++) { // Protocols
					Logger allResults = new Logger(subFolder, "All_" + protocols[p] + "-" + orderings[p] + ".csv");
					Logger results = new Logger(subFolder, protocols[p] + "-" + orderings[p] + ".csv");
					for (meetPerAgent = minMeetPerAgent ; meetPerAgent <= maxMeetPerAgent ; meetPerAgent += stepMeetPerAgent) {
						HashMap<String, Double> densityStatistics = new HashMap<String, Double>();
						for (instance = 1 ; instance <= nbInstance ; instance++) {
							String problem = problemsFolder + File.separator + meetPerAgent + File.separator + name + instance + ".xml.bz2";
							HashMap<String, Double> instanceStatistics = new HashMap<String, Double>();
							for (run = 1 ; run <= nbRun ; run++) {
								DisProblem disProb = new DisProblem(problem, protocols[p]);
								solver = new DisSolverSimulator(disProb);
								solver.setCutOffNCCCs(maxNCCCs);
								solver.addCommunicationMetric();
								solver.setMessageDelay(minURC, maxURC);
								solver.setCentralizedAO(AgentOrdering.getCentralizedAO(orderings[p]));
								solver.solve();
								if(keys == null ){
									keys = disProb.master.getStatisticsKeys();
								}
								HashMap<String, Double> runStatistics = solver.getStatistics();
								if (instance == 1 && run == 1 && meetPerAgent == minMeetPerAgent) {
									String s = ",";
									String header = Commands.MeetingScheduling + s;
									header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
									header += NB_MEETINGS + s + nbMeetings + s + NB_AGENTS + s + nbAgents + s ;
									header += MEETINGS_DURATION + s + meetingDuration + s + NB_DAYS + s + nbDays + s ;
									header += NB_HOURS + s + nbHours + s + DISPONIBILITY + s + disponibility + s ;
									header += X_axis + s + x_axis + s + Y_axis + s + y_axis + s;
									header += NB_INSTANCES+s+nbInstance + s + Commands.NbRun+s+nbRun+"\n";
									header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s ;
									header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "\n\n";
									header += NB_MEETINGS_PER_AGENT + s ;
									allResults.printToLogln(header + INSTANCE + s + Commands.NbRun + s + Tools.toString(keys));
									results.printToLogln(header + Tools.toString(keys));
								}
								allResults.printToLogln(meetPerAgent + "," + instance + "," + run + "," + Tools.statisticsToString(keys, runStatistics));
								instanceStatistics = Tools.merge(runStatistics, instanceStatistics);
								disProb.setAgents(null);
								System.gc();
							}
							Tools.average(instanceStatistics, nbRun);
							densityStatistics = Tools.merge(instanceStatistics, densityStatistics);
						}
						Tools.average(densityStatistics, nbInstance);
						results.printToLogln(meetPerAgent + "," + Tools.statisticsToString(keys, densityStatistics));
					}
					results.close();
					allResults.close();
					keys = null;
				} //Protocols
				String s = ",";
				ResultPrinter printer = new ResultPrinter(protocols, orderings, nbHeaderLine);
				printer.setFixKeys(fixKeys);
				ArrayList<String> paths = printer.printToGnuPlot(subFolder, "", commType);
				printer.plot(subFolder, paths);

				String header = Commands.MeetingScheduling + s;
				header += NB_MEETINGS + s + nbMeetings + s + NB_AGENTS + s + nbAgents + s ;
				header += MEETINGS_DURATION + s + meetingDuration + s + NB_DAYS + s + nbDays + s ;
				header += NB_HOURS + s + nbHours + s + DISPONIBILITY + s + disponibility + s ;
				header += X_axis + s + x_axis + s + Y_axis + s + y_axis + s;
				header += NB_INSTANCES+s+nbInstance + s + Commands.NbRun+s+nbRun+"\n";
				header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s ;
				header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "";
				printer.printAllResults(subFolder, header);
			}
		}
	}

	/* (non-Javadoc)
	 * @see gui.GuiSimulator#resumeWithoutSolver()
	 */
	@Override
	public void resumeWithoutSolver() {
		// TODO Auto-generated method stub
		System.err.println("I have to resume my experiments such that my solver is null");
	}
	
	public static boolean nextCombination(int[] combination, int n) {
		int i = combination.length - 1;
		++combination[i];
		while ((i > 0) && (combination[i] >= n - combination.length + 1 + i)) {
			--i;
			++combination[i];
		}
		/* Combination (n-k, n-k+1, ..., n) reached */ 
		if (combination[0] > n - combination.length)
			return false; /* No more combinations can be generated */
		/*
		 * comb now looks like (..., x, n, n, n, ..., n). Turn it into (..., x, x + 1, x + 2, ...)
		 */
		for (i = i + 1 ; i < combination.length ; ++i)
			combination[i] = combination[i - 1] + 1;
		return true;
	}
	
	public static ArrayList<int[]> getAllCombinations(int nbMeetings, int nbMeetingsPerAgent) {
		ArrayList<int[]> list = new ArrayList<int[]>();
		int combination[] = new int[nbMeetingsPerAgent];
		for (int i = 0 ; i < nbMeetingsPerAgent ; ++i) {
			combination[i] = i;
		}
		do {
			list.add(combination.clone());
		} while (nextCombination(combination, nbMeetings));
		return list;
	}
	
	public static void main(String[] args) {
		ArrayList<int[]> combi = null;
		long nbCombinaisons = 0;
		long current = System.currentTimeMillis();
		for (int i = 1 ; i < 21 ; i++) {
			combi = getAllCombinations(21, i);
			nbCombinaisons += combi.size();
//			System.out.println("---------------");
//			for (int[] is : combi) {
//				System.out.println(Tools.arrayToString(is));//TODO
//			}
		}
		System.out.println("Time: " + (System.currentTimeMillis()-current)+
				"\nnbCombinaisons: "+nbCombinaisons);
	}
}
