package benchmark.random;

import static util.parser.InstanceTokens.AGENT;
import static util.parser.InstanceTokens.AGENTS;
import static util.parser.InstanceTokens.ARITY;
import static util.parser.InstanceTokens.CONFLICTS;
import static util.parser.InstanceTokens.CONSTRAINT;
import static util.parser.InstanceTokens.CONSTRAINTS;
import static util.parser.InstanceTokens.CONSTRAINT_MODEL;
import static util.parser.InstanceTokens.DESCRIPTION;
import static util.parser.InstanceTokens.DOMAIN;
import static util.parser.InstanceTokens.DOMAINS;
import static util.parser.InstanceTokens.FORMAT;
import static util.parser.InstanceTokens.ID;
import static util.parser.InstanceTokens.INSTANCE;
import static util.parser.InstanceTokens.INSTANCES;
import static util.parser.InstanceTokens.MODEL;
import static util.parser.InstanceTokens.NAME;
import static util.parser.InstanceTokens.NB_AGENTS;
import static util.parser.InstanceTokens.NB_CONSTRAINTS;
import static util.parser.InstanceTokens.NB_DOMAINS;
import static util.parser.InstanceTokens.NB_INSTANCES;
import static util.parser.InstanceTokens.NB_RELATIONS;
import static util.parser.InstanceTokens.NB_TUPLES;
import static util.parser.InstanceTokens.NB_VALUES;
import static util.parser.InstanceTokens.NB_VARIABLES;
import static util.parser.InstanceTokens.PKC;
import static util.parser.InstanceTokens.PRESENTATION;
import static util.parser.InstanceTokens.REFERENCE;
import static util.parser.InstanceTokens.RELATION;
import static util.parser.InstanceTokens.RELATIONS;
import static util.parser.InstanceTokens.SCOPE;
import static util.parser.InstanceTokens.SEMANTICS;
import static util.parser.InstanceTokens.SIMPLE;
import static util.parser.InstanceTokens.TKC;
import static util.parser.InstanceTokens.TYPE;
import static util.parser.InstanceTokens.VARIABLE;
import static util.parser.InstanceTokens.VARIABLES;
import static util.parser.InstanceTokens.XDisCSP_1_0;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import dischoco.DisChoco;
import benchmark.ResultPrinter;
import gui.GuiSimulator;
import kernel.order.AgentOrdering;
import simulation.DisProblem;
import simulation.DisSolverSimulator;
import util.Commands;
import util.IntArray;
import util.Logger;
import util.Tools;
import util.parser.InstanceTokens;

public class RBDisCSPGenerator extends GuiSimulator {

	public static final String GRAPH_DENSITY = "graphDensity";
	public static final String TIGHTNESS = "tightness";
	public static final String P_1 = "p1";
	public static final String P_2 = "p2";
	public static final String MIN_P2 = "minP2";
	public static final String MAX_P2 = "maxP2";
	public static final String STEP_P2 = "stepP2";
	public static final String fixKeys = P_2;
	public static final String GuiPresentation = "GuiPresentation";
	
	/** number of variable in the problem */
	private int nbAgent;
	/** domain size of each agents */
	private int domainSize;
	/** the number of constraint*/
	private int nbConstraint;
	/** the Tightness (number of forbidden tuples)*/ 
	private int tightness;
	/** Graph density (ratio of number of constraint to n(n-1))*/ 
	private int minP1 = 20, maxP1 = 80, stepP1 = 10;
	/** Constraints tightness */ 
	private int minP2 = 10, maxP2 = 90, stepP2 = 10;
	private int nbInstances = 25;
	private Document document;
	private String path = ".";
	private boolean uniform = true;
	private int nbTest = 1000;
	
	public static int nbHeaderLine = 4;

	private IntArray constrainedAgents = new IntArray();
	private Random rand = new Random();
	private String tuples;
	private int PossibleCTs, PossibleNGs;
	/** CT means "constraint" */
	private int CTarray[] = null;
	/** NG means "nogood pair" */
	private int NGarray[] = null;
	/** totally known constraints model*/
	private boolean totallyKnown = true;
	private int p1 = 0, p2 = 0;
	private int instance, run;

	public RBDisCSPGenerator() {
		super();
	}

	public RBDisCSPGenerator(File directory, int nbInstances, int nbAgent, int domainSize) {
		this.path = directory.getPath();
		this.nbInstances = nbInstances;
		this.nbAgent = nbAgent;
		this.domainSize = domainSize;
	}

	/**
	 * sets the model of constraints
	 * 
	 * @param tkc
	 *            </br> <b><i>= true</i></b> to generate totally known constraints. </br> <b><i>= false</i></b> to generate partially known
	 *            constraints.
	 */
	public void setModel(boolean tkc) {
		this.totallyKnown = tkc; // totally known constraints model
	}

	public void setProperties(double minP1, double maxP1, double stepP1, double minP2, double maxP2, double stepP2) {
		this.minP1 = (int) (minP1 * 100);
		this.maxP1 = (int) (maxP1 * 100);
		this.stepP1 = (int) (stepP1 * 100);
		this.minP2 = (int) (minP2 * 100);
		this.maxP2 = (int) (maxP2 * 100);
		this.stepP2 = (int) (stepP2 * 100);
	}

	public void setP1(int minP1, int maxP1, int stepP1) {
		this.minP1 = minP1;
		this.maxP1 = maxP1;
		this.stepP1 = stepP1;
	}

	public void setP2(int minP2, int maxP2, int stepP2) {
		this.minP2 = minP2;
		this.maxP2 = maxP2;
		this.stepP2 = stepP2;
	}

	/**
	 * @param file
	 */
	void save(String file) {
		try {
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			out.output(this.document, new FileOutputStream(file));
			Process p = Runtime.getRuntime().exec("bzip2 " + file);
			p.waitFor();
			p.exitValue();
			p.destroy();
		} catch (Exception e) {}
	}

	/**
	 * 
	 * @param nbInstance
	 * @return
	 */
	private Element getPresentation(int nbInstance) {
		Element presentation = new Element(PRESENTATION);
		presentation.setAttribute(NAME, INSTANCE + (nbInstance + 1));
		presentation.setAttribute(TYPE, Commands.RandomDisCSP);
		presentation.setAttribute(MODEL, SIMPLE);
		if (this.totallyKnown) {
			presentation.setAttribute(CONSTRAINT_MODEL, TKC);
		} else {
			presentation.setAttribute(CONSTRAINT_MODEL, PKC);
		}
		presentation.setAttribute(FORMAT, XDisCSP_1_0);
		return presentation;
	}

	/**
	 * 
	 * @param nbAgents
	 * @return
	 */
	private Element getAgents(int nbAgents) {
		Element agents = new Element(AGENTS);
		agents.setAttribute(NB_AGENTS, "" + nbAgents);
		for (int id = 1 ; id <= nbAgents ; id++) {
			Element agent = new Element(AGENT);
			agent.setAttribute(NAME, "A" + id);
			agent.setAttribute(ID, "" + id);
			agent.setAttribute(DESCRIPTION, AGENT + " " + id);
			agents.addContent(agent);
		}
		return agents;
	}

	/**
	 * 
	 * @param domainSize
	 * @return
	 */
	private Element getDomaines(int domainSize) {
		Element domains = new Element(DOMAINS);
		domains.setAttribute(NB_DOMAINS, "1");
		Element domain = new Element(DOMAIN);
		domain.setAttribute(NAME, "D1");
		domain.setAttribute(NB_VALUES, "" + domainSize);
		domain.setText("0.." + (domainSize - 1));
		domains.addContent(domain);
		return domains;
	}

	/**
	 * 
	 * @param nbVariables
	 * @return
	 */
	private Element getVariables(int nbVariables) {
		Element variables = new Element(VARIABLES);
		variables.setAttribute(NB_VARIABLES, "" + nbVariables);
		for (int id = 1 ; id <= nbVariables ; id++) {
			Element variable = new Element(VARIABLE);
			variable.setAttribute(AGENT, "A" + id);
			variable.setAttribute(NAME, "X" + id + ".0");
			variable.setAttribute(ID, "0");
			variable.setAttribute(DOMAIN, "D1");
			variable.setAttribute(DESCRIPTION, VARIABLE + " " + id + ".0");
			variables.addContent(variable);
		}
		return variables;
	}
	
	/**
	 * @return
	 */
	private Element getGuiPresentation(int nbInstance) {
		Element presentation = new Element(GuiPresentation);
		presentation.setAttribute(TYPE, Commands.RandomDisCSP);
		presentation.setAttribute(NAME, INSTANCE + (nbInstance + 1));
		presentation.setAttribute(MODEL, SIMPLE);
		presentation.setAttribute(NB_AGENTS, ""+this.nbAgent);
		presentation.setAttribute(Commands.DomainSize, ""+this.domainSize);
		presentation.setAttribute(GRAPH_DENSITY, ""+this.p1);
		presentation.setAttribute(TIGHTNESS, ""+this.p2);
		presentation.setAttribute(NB_CONSTRAINTS, ""+this.nbConstraint);
		presentation.setAttribute(NB_TUPLES, "" + this.tightness);
		return presentation;
	}

	/**
	 * 
	 */
	public void generate() {
		Element experiments = new Element(Commands.Experiments);
		Document confdocument = new Document(experiments);
		if (this.totallyKnown) {
			experiments.setAttribute(TYPE, Commands.RandomTKC);
		} else {
			experiments.setAttribute(TYPE, Commands.RandomPKC);
		}
		Element instances = new Element(INSTANCES);
		experiments.addContent(instances);
		if (this.totallyKnown) {
			instances.setAttribute(TYPE, Commands.RandomTKC);
		} else {
			instances.setAttribute(TYPE, Commands.RandomPKC);
		}
		instances.setAttribute(NB_AGENTS, "" + this.nbAgent);
		instances.setAttribute(Commands.DomainSize, "" + this.domainSize);
		String name = Commands.RandomPKC + "_" + this.nbAgent + "_" + this.domainSize + "_";
		if (this.totallyKnown) {
			name = Commands.RandomTKC + "_" + this.nbAgent + "_" + this.domainSize + "_";
		}
		instances.setAttribute(NAME, name);
		for (p1 = this.minP1 ; p1 <= this.maxP1 ; p1 += this.stepP1) {
			String subDensity = P_1 + "_" + "0." + ((p1 < 10)? "0" + p1 : p1);
			this.nbConstraint = this.nbAgent * (this.nbAgent - 1) * p1 / 200;
			File folder = new File(this.path + File.separator + subDensity);
			if (!folder.exists()) { folder.mkdir(); }
			for (p2 = this.minP2 ; p2 <= this.maxP2 ; p2 += this.stepP2) {
				String subTightness = "0." + ((p2 < 10)? "0" + p2 : p2);// Commands.p2+"_"+
				this.tightness = this.domainSize * this.domainSize * p2 / 100;
				File subFolder = new File(folder.getPath() + File.separator + subTightness);
				if (!subFolder.exists()) {
					subFolder.mkdir();
				}
				for (int i = 0 ; i < this.nbInstances ; i++) {
					this.document = new Document(this.generateInstance(i));
					this.save(subFolder.getPath() + File.separator + name + (i + 1) + ".xml");
				}
			}
			Element graphDensity = new Element(GRAPH_DENSITY);
			graphDensity.setAttribute(NAME, subDensity);
			graphDensity.setAttribute(P_1, "0." + ((p1 < 10)? "0" + p1 : p1));
			Element tightness = new Element(TIGHTNESS);
			tightness.setAttribute(MIN_P2, "0." + ((this.minP2 < 10)? "0" + this.minP2 : this.minP2));
			tightness.setAttribute(MAX_P2, "0." + ((this.maxP2 < 10)? "0" + this.maxP2 : this.maxP2));
			tightness.setAttribute(STEP_P2, "0." + ((this.stepP2 < 10)? "0" + this.stepP2 : this.stepP2));
			tightness.setAttribute(InstanceTokens.NB_INSTANCES, this.nbInstances + "");
			graphDensity.addContent(tightness);
			instances.addContent(graphDensity);
		}
		try {
			String confName = Commands.Experiments + "_" + this.nbAgent + "_" + this.domainSize + "_" + this.nbInstances;
			File conf = new File(this.path + File.separator + confName + ".xml");
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			out.output(confdocument, new FileOutputStream(conf));
		} catch (Exception e) {}
	}

	/**
	 * 
	 * @param nbI
	 * @return
	 */
	private Element generateInstance(int nbInstance) {
		Element instance = new Element(INSTANCE);
		instance.addContent(this.getPresentation(nbInstance));
		instance.addContent(this.getAgents(this.nbAgent));
		instance.addContent(this.getDomaines(this.domainSize));
		instance.addContent(this.getVariables(this.nbAgent));
		/*************************************************************************/
		this.PossibleCTs = this.nbAgent * (this.nbAgent - 1) / 2;
		/* Create an array for each possible value pair. */
		this.CTarray = new int[this.PossibleCTs];
		/*
		 * Initialize the CTarray. Each entry has one variable in the high two bytes, and the other in the low two bytes.
		 */
		/* Create an array for each possible binary constraint. */
		for (int i = 0 , var1 = 0 ; var1 < (this.nbAgent - 1) ; ++var1) {
			for (int var2 = var1 + 1 ; var2 < this.nbAgent ; ++var2) {
				this.CTarray[i++] = (var1 << 16) | var2;
			}
		}
		this.PossibleNGs = this.domainSize * this.domainSize;
		// NGarray = (unsigned long*) malloc(PossibleNGs * 4);
		this.NGarray = new int[this.PossibleNGs];
		/* Initialize the NGarray. */
		for (int i = 0 ; i < this.PossibleNGs ; i++) {
			this.NGarray[i] = i;
		}
		/*************************************************************************/
		this.rand.setSeed(System.currentTimeMillis());
		this.constrainedAgents = new IntArray();
		if (this.totallyKnown) {
			this.generateTotallyKnownInstance(instance);
		} else {
			this.generatePartiallyKnownInstance(instance);
		}
		instance.addContent(this.getGuiPresentation(nbInstance));
		return instance;
	}

	private void generateTotallyKnownInstance(Element instance) {
		Element constraints = null;
		Element relations = null;
		int selectedCT, selectedNG;
		int vr1, vr2, vl1, vl2;
		/* Select C constraints. */
		int test = 0;
		do {
			constraints = new Element(CONSTRAINTS);
			constraints.setAttribute(NB_CONSTRAINTS, "" + this.nbConstraint);
			relations = new Element(RELATIONS);
			relations.setAttribute(NB_RELATIONS, "" + this.nbConstraint);
			this.constrainedAgents = new IntArray();
			for (int c = 0 ; c < this.nbConstraint ; c++) {
				/*
				 * Choose a random number between c and PossibleCTs - 1, inclusive.
				 */
				int r = c + (int) (this.rand.nextFloat() * (PossibleCTs - c));
				/* Swap elements [c] and [r]. */
				selectedCT = this.CTarray[r];
				this.CTarray[r] = this.CTarray[c];
				this.CTarray[c] = selectedCT;
				/* Broadcast the constraint. */
				Element constraint = new Element(CONSTRAINT);
				constraint.setAttribute(NAME, "C" + c);
				constraint.setAttribute(MODEL, Commands.TKC);
				constraint.setAttribute(ARITY, "2");
				/* Broadcast the constraint. */
				vr1 = (int) (this.CTarray[c] >> 16) + 1;
				vr2 = (int) (this.CTarray[c] & 0x0000FFFF) + 1;
				this.constrainedAgents.add(vr1);
				this.constrainedAgents.add(vr2);
				constraint.setAttribute(SCOPE, "X" + vr1 + ".0" + " X" + vr2 + ".0");
				constraint.setAttribute(REFERENCE, "R" + c);
				/* For each constraint, select T illegal value pairs. */
				Element relation = new Element(RELATION);
				relation.setAttribute(NAME, "R" + c);
				relation.setAttribute(ARITY, "2");
				relation.setAttribute(NB_TUPLES, "" + this.tightness);
				relation.setAttribute(SEMANTICS, CONFLICTS);
				tuples = "";
				/* Select T incompatible pairs. */
				for (int t = 0 ; t < this.tightness ; ++t) {
					/*
					 * Choose a random number between t and PossibleNGs - 1, inclusive.
					 */
					r = t + (int) (rand.nextFloat() * (PossibleNGs - t));
					selectedNG = this.NGarray[r];
					this.NGarray[r] = this.NGarray[t];
					this.NGarray[t] = selectedNG;

					/* Broadcast the nogood value pair. */
					vl1 = (int) (NGarray[t] / this.domainSize);
					vl2 = (int) (NGarray[t] % this.domainSize);
					tuples = tuples.concat(vl1 + " " + vl2);
					if (t < this.tightness - 1) {
						tuples = tuples.concat("|");
					}
				}
				relation.setText(tuples);
				constraints.addContent(constraint);
				relations.addContent(relation);
			}
			if (test++ >= this.nbTest && this.uniform) { // uniform
				throw new Error("\nImpossible to generate an uniform Random Problem");
			}
		} while (this.uniform && this.constrainedAgents.size() < this.nbAgent);
		instance.addContent(constraints);
		instance.addContent(relations);
	}

	private void generatePartiallyKnownInstance(Element instance) {
		Element constraints = null;
		Element relations = null;
		int selectedCT, selectedNG;
		int vr1, vr2, vl1, vl2;
		/* Select C constraints. */
		int test = 0;
		do {
			constraints = new Element(CONSTRAINTS);
			constraints.setAttribute(NB_CONSTRAINTS, "" + 2 * this.nbConstraint);
			relations = new Element(RELATIONS);
			relations.setAttribute(NB_RELATIONS, "" + 2 * this.nbConstraint);
			this.constrainedAgents = new IntArray();
			/* Select C constraints. */
			for (int c = 0 ; c < this.nbConstraint ; c++) {
				/*
				 * Choose a random number between c and PossibleCTs - 1, inclusive.
				 */
				int r = c + (int) (this.rand.nextFloat() * (this.PossibleCTs - c));
				/* Swap elements [c] and [r]. */
				selectedCT = this.CTarray[r];
				this.CTarray[r] = this.CTarray[c];
				this.CTarray[c] = selectedCT;

				/* Broadcast the constraint. */
				vr1 = (int) (this.CTarray[c] >> 16) + 1;
				vr2 = (int) (this.CTarray[c] & 0x0000FFFF) + 1;
				this.constrainedAgents.add(vr1);
				this.constrainedAgents.add(vr2);
				Element constraint1 = new Element(CONSTRAINT);
				constraint1.setAttribute(NAME, "C" + (2 * c));
				constraint1.setAttribute(ARITY, "2");
				constraint1.setAttribute(MODEL, Commands.PKC); // Partially
				// Known
				// Constraints
				constraint1.setAttribute(SCOPE, "X" + vr1 + ".0 " + "X" + vr2 + ".0");
				constraint1.setAttribute(REFERENCE, "R" + (2 * c));
				constraints.addContent(constraint1);

				Element constraint2 = new Element(CONSTRAINT);
				constraint2.setAttribute(NAME, "C" + ((2 * c) + 1));
				constraint2.setAttribute(ARITY, "2");
				constraint2.setAttribute(MODEL, Commands.PKC); // Partially
				// Known
				// Constraints
				constraint2.setAttribute(SCOPE, "X" + vr2 + ".0 " + "X" + vr1 + ".0");
				constraint2.setAttribute(REFERENCE, "R" + ((2 * c) + 1));
				constraints.addContent(constraint2);
				/* For each constraint, select T illegal value pairs. */

				/* Initialize the NGarray. */
				for (int i = 0 ; i < (this.domainSize * this.domainSize) ; ++i) {
					this.NGarray[i] = i;
				}
				int T1 = this.tightness / 2;
				Element relation1 = new Element(RELATION);
				relation1.setAttribute(NAME, "R" + ((2 * c)));
				relation1.setAttribute(ARITY, "2");
				relation1.setAttribute(NB_TUPLES, "" + T1);
				relation1.setAttribute(SEMANTICS, CONFLICTS);
				String tuples1 = "";
				/* Select T incompatible pairs. */
				for (int t = 0 ; t < T1 ; t++) {
					/*
					 * Choose a random number between t and PossibleNGs - 1, inclusive.
					 */
					r = t + (int) (this.rand.nextFloat() * (this.PossibleNGs - t));
					selectedNG = this.NGarray[r];
					this.NGarray[r] = this.NGarray[t];
					this.NGarray[t] = selectedNG;

					/* Broadcast the nogood value pair. */
					vl1 = (int) (this.NGarray[t] / this.domainSize);
					vl2 = (int) (this.NGarray[t] % this.domainSize);
					tuples1 = tuples1.concat(vl1 + " " + vl2);
					if (t < T1 - 1) {
						tuples1 = tuples1.concat("|");
					}
				}
				relation1.setText(tuples1);
				relations.addContent(relation1);

				Element relation2 = new Element(RELATION);
				relation2.setAttribute(NAME, "R" + ((2 * c) + 1));
				relation2.setAttribute(ARITY, "2");
				relation2.setAttribute(NB_TUPLES, "" + (this.tightness - T1));
				relation2.setAttribute(SEMANTICS, CONFLICTS);
				String tuples2 = "";
				/* Select T incompatible pairs. */
				for (int t = T1 ; t < this.tightness ; t++) {
					/*
					 * Choose a random number between t and PossibleNGs - 1, inclusive.
					 */
					r = t + (int) (rand.nextFloat() * (PossibleNGs - t));
					selectedNG = NGarray[r];
					NGarray[r] = NGarray[t];
					NGarray[t] = selectedNG;

					/* Broadcast the nogood value pair. */
					vl1 = (int) (NGarray[t] / this.domainSize);
					vl2 = (int) (NGarray[t] % this.domainSize);
					tuples2 = tuples2.concat(vl2 + " " + vl1);
					if (t < this.tightness - 1) {
						tuples2 = tuples2.concat("|");
					}
				}
				relation2.setText(tuples2);
				relations.addContent(relation2);
			}
			if (test++ >= this.nbTest && this.uniform) { // uniform
				throw new Error("\nImpossible to generate an uniform Random Problem");
			}
		} while (this.uniform && this.constrainedAgents.size() < this.nbAgent);
		instance.addContent(constraints);
		instance.addContent(relations);
	}

	/**
	 * 
	 * @param str
	 * @return
	 */
	public static boolean isInt(String str) {
		if (str.length() > 0 && str.charAt(0) != '+' && str.charAt(0) != '-' && !(str.charAt(0) >= '0' && str.charAt(0) <= '9'))
			return false;
		for (int i = 1 ; i < str.length() ; i++) {
			char ch = str.charAt(i);
			if (ch < '0' || ch > '9')
				return false;
		}
		return true;
	}

	@SuppressWarnings("unchecked")
	public static boolean lunchXP(File file) {
		int nbInstance, minP2, maxP2, stepP2;
		String p1;
		ArrayList<String> keys = null;
		String filePath = file.getParent();
		String directory = "";
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(file);
		} catch (Exception e) {}
		Element xp = document.getRootElement();
		Element instances = xp.getChild(INSTANCES);
		List<Element> communications = xp.getChild(Commands.Communications).getChildren();
		List<Element> algorithms = xp.getChild(Commands.Algorithms).getChildren();
		String[] protocols = new String[algorithms.size()];
		String[] orderings = new String[algorithms.size()];
		int a = 0;
		Iterator<Element> it = algorithms.iterator();
		while (it.hasNext()) {
			Element algorithm = it.next();
			protocols[a] = algorithm.getText();
			orderings[a] = algorithm.getAttributeValue(Commands.AO);
			a++;
		}
		it = communications.iterator();
		int nbAgents = new Integer(instances.getAttributeValue(NB_AGENTS));
		int domainSize = new Integer(instances.getAttributeValue(Commands.DomainSize));
		while (it.hasNext()) {
			// /////////////////////////////////////////////////////////////////
			Element comm = it.next();
			String commType = comm.getAttributeValue(TYPE);
			String channelType = comm.getAttributeValue(Commands.channelType);
			int minURC = new Integer(comm.getAttributeValue(Commands.MinURC));
			int maxURC = new Integer(comm.getAttributeValue(Commands.MaxURC));
			int nbRun = new Integer(comm.getAttributeValue(Commands.NbRun));
			long maxNCCCs = new Long(comm.getAttributeValue(Commands.MaxNCCCs));
			String name = instances.getAttribute(NAME).getValue();
			List<Element> graphDensities = instances.getChildren();
			for (int d = 0 ; d < graphDensities.size() ; d++) {
				Element graphDensity = graphDensities.get(d);
				p1 = graphDensity.getAttributeValue(P_1);
				Element tightness = graphDensity.getChild(TIGHTNESS);
				minP2 = (int) ((new Double(tightness.getAttributeValue(MIN_P2))) * 100);
				maxP2 = (int) ((new Double(tightness.getAttributeValue(MAX_P2))) * 100);
				stepP2 = (int) ((new Double(tightness.getAttributeValue(STEP_P2))) * 100);
				nbInstance = new Integer(tightness.getAttributeValue(NB_INSTANCES));
				directory = filePath + File.separator + Commands.Results+"_"+nbAgents+"_"+ nbInstance;
				try {
					File f = new File(directory);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				directory += File.separator + commType + "_" + channelType;
				try {
					File f = new File(directory);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				String subFolder = directory + File.separator + graphDensity.getAttributeValue(NAME);
				String problemsFolder = filePath + File.separator + graphDensity.getAttributeValue(NAME);
				try {
					File f = new File(subFolder);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				for (int p = 0 ; p < protocols.length ; p++) { // Protocols
					Logger allResuls = new Logger(subFolder, "All_" + protocols[p] + "-" + orderings[p] + ".csv");
					Logger results = new Logger(subFolder, protocols[p] + "-" + orderings[p] + ".csv");
					for (int p2 = minP2 ; p2 <= maxP2 ; p2 += stepP2) {
						String _p2 = (p2 < 10)? "0.0" + p2 : "0." + p2;
						HashMap<String, Double> densityStatistics = new HashMap<String, Double>();
						for (int i = 1 ; i <= nbInstance ; i++) {
							String problem = problemsFolder + File.separator + _p2 + File.separator + name + i + ".xml.bz2";
							HashMap<String, Double> instanceStatistics = new HashMap<String, Double>();
							for (int r = 1 ; r <= nbRun ; r++) {
								DisProblem disProb = new DisProblem(problem, protocols[p]);
								DisSolverSimulator solver = new DisSolverSimulator(disProb);
								solver.setCutOffNCCCs(maxNCCCs);
								solver.addCommunicationMetric();
								solver.setMessageDelay(minURC, maxURC);
								solver.setCentralizedAO(AgentOrdering.getCentralizedAO(orderings[p]));
								solver.CheckTheReturnedSolution(true);
								solver.solve();
								if (keys == null) {
									keys = disProb.master.getStatisticsKeys();
								}
								HashMap<String, Double> runStatistics = solver.getStatistics();
								if (i == 1 && r == 1 && p2 == minP2) {
									String s = ", ";
									String header = Commands.RandomDisCSP + s;
									header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
									header += NB_AGENTS + s + nbAgents + s + Commands.DomainSize + s + domainSize + s ;
									header += P_1 + s + p1 + s + NB_INSTANCES + s + nbInstance+ "\n";
									header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s ;
									header += Commands.MinURC+ s + minURC + s + Commands.MaxURC + s + maxURC + "\n\n";
									header += P_2 + s + INSTANCE + s + Commands.NbRun + s + Tools.toString(keys);
									allResuls.printToLogln(header);
									header = Commands.RandomDisCSP + s;
									header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
									header += NB_AGENTS + s + nbAgents + s + Commands.DomainSize + s + domainSize + s ;
									header += P_1 + s + p1 + s + NB_INSTANCES + s + nbInstance+ "\n";
									header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s ;
									header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "\n\n";
									header += P_2 + s + Tools.toString(keys);
									results.printToLogln(header);
								}
								allResuls.printToLogln(_p2 + ", " + i + ", " + r + ", " + Tools.statisticsToString(keys, runStatistics));
								instanceStatistics = Tools.merge(runStatistics, instanceStatistics);
								disProb.clear();
								disProb.setAgents(null);
								disProb = null;
								System.gc();
							}
							Tools.average(instanceStatistics, nbRun);
							densityStatistics = Tools.merge(instanceStatistics, densityStatistics);
						}
						Tools.average(densityStatistics, nbInstance);
						results.printToLogln(_p2 + ", " + Tools.statisticsToString(keys, densityStatistics));
					}
					results.close();
					allResuls.close();
					keys = null;
				} // protocols
				ResultPrinter printer = new ResultPrinter(protocols, orderings, nbHeaderLine);
				printer.setFixKeys(fixKeys);
				ArrayList<String> paths = printer.printToGnuPlot(subFolder, "", commType);
				printer.plot(subFolder, paths);
				String s = ",";
				String header = Commands.RandomDisCSP + s;
				header += NB_AGENTS + s + nbAgents + s + Commands.DomainSize + s + domainSize + "\n";
				header += "\n#";
				header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s;
				header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + s + P_1 + s + p1 + s + NB_INSTANCES + s + nbInstance;
				printer.printAllResults(subFolder, header);
				
			} // density
		}
		return true;
	}

	@SuppressWarnings("unchecked")
	@Override
	public void lunch() {
		System.out.println("lunch");
		String p1;
		ArrayList<String> keys = null;
		String filePath = DisChoco.filePath.getParent();
		String directory = "";
		Document document = null;
		SAXBuilder sxb = new SAXBuilder();
		try {
			document = sxb.build(DisChoco.filePath);
		} catch (Exception e) {}
		Element xp = document.getRootElement();
		Element instances = xp.getChild(INSTANCES);
		List<Element> communications = xp.getChild(Commands.Communications).getChildren();
		List<Element> algorithms = xp.getChild(Commands.Algorithms).getChildren();
		String[] protocols = new String[algorithms.size()];
		String[] orderings = new String[algorithms.size()];
		int a = 0;
		Iterator<Element> it = algorithms.iterator();
		while (it.hasNext()) {
			Element algorithm = it.next();
			protocols[a] = algorithm.getText();
			orderings[a] = algorithm.getAttributeValue(Commands.AO);
			a++;
		}
		it = communications.iterator();
		int nbAgents = new Integer(instances.getAttributeValue(NB_AGENTS));
		int domainSize = new Integer(instances.getAttributeValue(Commands.DomainSize));
		while (it.hasNext()) {
			// /////////////////////////////////////////////////////////////////
			Element comm = it.next();
			String commType = comm.getAttributeValue(TYPE);
			String channelType = comm.getAttributeValue(Commands.channelType);
			int minURC = new Integer(comm.getAttributeValue(Commands.MinURC));
			int maxURC = new Integer(comm.getAttributeValue(Commands.MaxURC));
			int nbRun = new Integer(comm.getAttributeValue(Commands.NbRun));
			long maxNCCCs = new Long(comm.getAttributeValue(Commands.MaxNCCCs));
			String name = instances.getAttribute(NAME).getValue();
			List<Element> graphDensities = instances.getChildren();
			for (int d = 0 ; d < graphDensities.size() ; d++) {
				Element graphDensity = graphDensities.get(d);
				p1 = graphDensity.getAttributeValue(P_1);
				Element tightness = graphDensity.getChild(TIGHTNESS);
				minP2 = (int) ((new Double(tightness.getAttributeValue(MIN_P2))) * 100);
				maxP2 = (int) ((new Double(tightness.getAttributeValue(MAX_P2))) * 100);
				stepP2 = (int) ((new Double(tightness.getAttributeValue(STEP_P2))) * 100);
				nbInstances = new Integer(tightness.getAttributeValue(NB_INSTANCES));
				directory = filePath + File.separator + Commands.Results+"_"+nbAgents+"_"+nbInstances;
				try {
					File f = new File(directory);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				directory += File.separator + commType + "_" + channelType;
				try {
					File f = new File(directory);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				String subFolder = directory + File.separator + graphDensity.getAttributeValue(NAME);
				String problemsFolder = filePath + File.separator + graphDensity.getAttributeValue(NAME);
				try {
					File f = new File(subFolder);
					if (!f.exists()) { f.mkdir(); }
				} catch (Exception e) {}
				for (int p = 0 ; p < protocols.length ; p++) { // Protocols
					Logger allResuls = new Logger(subFolder, "All_" + protocols[p] + "-" + orderings[p] + ".csv");
					Logger results = new Logger(subFolder, protocols[p] + "-" + orderings[p] + ".csv");
					for (p2 = minP2 ; p2 <= maxP2 ; p2 += stepP2) {
						String _p2 = (p2 < 10)? "0.0" + p2 : "0." + p2;
						HashMap<String, Double> densityStatistics = new HashMap<String, Double>();
						for (instance = 1 ; instance <= nbInstances ; instance++) {
							String problem = problemsFolder + File.separator + _p2 + File.separator + name + instance + ".xml.bz2";
							HashMap<String, Double> instanceStatistics = new HashMap<String, Double>();
							for (run = 1 ; run <= nbRun ; run++) {
								DisProblem disProb = new DisProblem(problem, protocols[p]);
								solver = new DisSolverSimulator(disProb);
								solver.setCutOffNCCCs(maxNCCCs);
								solver.addCommunicationMetric();
								solver.setMessageDelay(minURC, maxURC);
								solver.setCentralizedAO(AgentOrdering.getCentralizedAO(orderings[p]));
								solver.CheckTheReturnedSolution(true);
								solver.solve();
								if (keys == null ) {
									keys = disProb.master.getStatisticsKeys();
								}
								HashMap<String, Double> runStatistics = solver.getStatistics();
								solver = null;
								if (instance == 1 && run == 1 && p2 == minP2) {
									String s = ",";
									String header = Commands.RandomDisCSP + s;
									header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
									header += NB_AGENTS + s + nbAgents + s + Commands.DomainSize + s + domainSize + s ;
									header += P_1 + s + p1 + s + NB_INSTANCES + s + nbInstances+ "\n";
									header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s ; 
									header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "\n\n";
									header += P_2 + s + INSTANCE + s + Commands.NbRun + s + Tools.toString(keys);
									allResuls.printToLogln(header);
									header = Commands.RandomDisCSP + s;
									header += Commands.Algorithm + s + protocols[p] + s + Commands.AO + s + orderings[p] + "\n";
									header += NB_AGENTS + s + nbAgents + s + Commands.DomainSize + s + domainSize + s ;
									header += P_1 + s + p1 + s + NB_INSTANCES + s + nbInstances+ "\n";
									header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s ;
									header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + "\n\n";
									header += P_2 + s + Tools.toString(keys);
									results.printToLogln(header);
								}
								allResuls.printToLogln(_p2 + "," + instance + "," + run + "," + Tools.statisticsToString(keys, runStatistics));
								instanceStatistics = Tools.merge(runStatistics, instanceStatistics);
								disProb.clear();
								disProb.setAgents(null);
								disProb = null;
								System.gc();
							}
							Tools.average(instanceStatistics, nbRun);
							densityStatistics = Tools.merge(instanceStatistics, densityStatistics);
						}
						Tools.average(densityStatistics, nbInstances);
						results.printToLogln(_p2 + "," + Tools.statisticsToString(keys, densityStatistics));
					}
					results.close();
					allResuls.close();
					keys = null;
				} // protocols
				ResultPrinter printer = new ResultPrinter(protocols, orderings, nbHeaderLine);
				printer.setFixKeys(fixKeys);
				ArrayList<String> paths = printer.printToGnuPlot(subFolder, "", commType);
				printer.plot(subFolder,paths);
				String s = ",";
				String header = Commands.RandomDisCSP + s;
				header += NB_AGENTS + s + nbAgents + s + Commands.DomainSize + s + domainSize + "\n";
				header += "\n#";
				header += Commands.channelType + s + channelType + s + Commands.Communication + s + commType + s;
				header += Commands.MinURC + s + minURC + s + Commands.MaxURC + s + maxURC + s + P_1 + s + p1 + s + NB_INSTANCES + s + nbInstances;
				printer.printAllResults(subFolder, header);
			} // density
		}
	}

	/* (non-Javadoc)
	 * @see gui.GuiSimulator#resumeWithoutSolver()
	 */
	@Override
	public void resumeWithoutSolver() {
		// TODO Auto-generated method stub
		System.err.println("I have to resume my experiments such that my solver is null");
	}
}
