/************************************************************************

 **************************************************************************/

package kernel.order;

import java.util.HashMap;

import kernel.AbstractMaster;
import kernel.StopException;
import kernel.com.Message;
import kernel.com.MessageHandler;
import util.Commands;
import util.IntArray;
import util.graph.Graph;

/**
 * An abstract class for all implementation of centralized agent ordering
 */
abstract public class CentralizedAO extends MessageHandler {

	/**
	 * the master to monitor its execution by the observer
	 */

	protected AbstractMaster master;

	protected Graph<Integer> graph = new Graph<Integer>();

	/**
	 * Set the master to observe
	 * 
	 * @param master
	 */

	public void setMaster(AbstractMaster master) {
		this.master = master;
	}

	/**
	 * A map to store all received agent complexity (complexity by ID)
	 */
	protected HashMap<Integer, AgentComplexity> agentsComplexity = null;
	/**
	 * A field stores the current number of received complexity. When this number becomes equal to the total number of agents construction of the
	 * agents order begins.
	 */
	protected int tmpRcvdMsgCounter = 0;

	protected IntArray order = new IntArray();

	/**
	 * Constructs a centralized agent ordering
	 */
	public CentralizedAO() {
		super(null, Commands.AO);
		this.agentsComplexity = new HashMap<Integer, AgentComplexity>();
	}

	/**
	 * Called to build agents order using the received complexities
	 * 
	 * @return IntArray an array representing the resulting order
	 */
	abstract public IntArray buildAgentsOrder();

	public IntArray getOrder() {
		return this.order;
	}

	/**
	 * Adds an agent complexity
	 * 
	 * @param agId
	 *            the ID of the agent with the given complexity
	 * @param complexity
	 *            the complexity of the agent with the given ID
	 */
	public void addAgentComplexity(int agId, AgentComplexity complexity) {
		this.agentsComplexity.put(agId, complexity);
	}

	/**
	 * Returns the map of all received complexity
	 */
	public HashMap<Integer, AgentComplexity> getAgentsComplexity() {
		return this.agentsComplexity;
	}

	/**
	 * {@inheritDoc} Handles a message ORDER. It adds the attached complexity to the map and increment the counter {@link #tmpRcvdMsgCounter}. When
	 * the counter becomes equals to the number of known agents it start the construction of the order and broadcast the resulting order to all
	 * agents.
	 * 
	 */
	@Override
	public void processMessage(Message msg) throws StopException {
		if (msg.getType().equals(Commands.AO)) {
			AgentComplexity cmpx = (AgentComplexity) msg.get(Commands.AGENT_COMPLEXITY);
			this.addAgentComplexity(msg.getSenderId(), cmpx);
			this.tmpRcvdMsgCounter++;
			// if(Logger.LOG_DEBUG)
			// this.master.log(" number of rcvd msg " + this.tmpRcvdMsgCounter
			// + "  the number of agent is " + this.master.getNumAgents(), Logger.LEVEL_TRACE_MESSAGE);
			if (this.tmpRcvdMsgCounter == this.master.getNumAgents()) {
				for (int v : agentsComplexity.keySet()) {
					graph.addVertex(new Integer(v));
					AgentComplexity neig = agentsComplexity.get(v);
					for (int w : neig.getNeighbors()) {
						graph.addEdge(v, w);
					}
				}
				this.order = this.buildAgentsOrder();
				this.broadcastOrder();
			}
		}
	}

	/**
	 * Called to broadcast the order.
	 * 
	 * @param order
	 *            an array representing the agent ordering
	 */
	public void broadcastOrder() {
		Message msg = this.master.createMessage(Commands.AO);
		msg.addContents(Commands.ORDER, order);
		msg.addContents(Commands.GRAPH, Graph.toMsg(graph));
		this.master.broadcast(msg, AbstractMaster.MasterId);
	}

}
