package simple.dcsp.afcfamily.afc.domdeg;

import java.util.HashMap;
import java.util.Iterator;

import kernel.AbstractAgent;
import kernel.StopException;
import kernel.com.Message;
import kernel.com.MessageHandler;
import kernel.order.AgentOrdering;
import simple.dcsp.afcfamily.afc.AFCAgent;
import util.Commands;
import util.Instantiation;
import util.IntArray;

public class AFCDomDegAgent extends AFCAgent {

	long myCounter = 0;
	HashMap<Integer, Long> counters;
	HashMap<Integer, Integer> domainSizes;

	OrderMsgHandler orderMsgH = null;

	private int heuristic = AFCMinDomMaster.Dom_Dyn_Deg;

	public AFCDomDegAgent(int agentId, String agName, String description, int heuristic) {
		super(agentId, agName, description);
		this.domainSizes = new HashMap<Integer, Integer>();
		this.counters = new HashMap<Integer, Long>();
		this.orderMsgH = new OrderMsgHandler(this);
		this.agentOrder = new AFCDomDegAO(this);
		this.heuristic = heuristic;
	}

	// ==========================================================================
	protected void receiveCPA(Instantiation cpa, long sc, boolean fromBackMsg) throws StopException {
		if (sc > this.stepCounter) {
			if (this.updateAgentOrdering(cpa, !fromBackMsg)) {
				this.agentView.clearExcept(cpa);
				// this.agentView.priorityMerge(notOk);
				this.agentViewConsistent = true;
			}
		}
		super.receiveCPA(cpa, sc, fromBackMsg);
	}

	// ==========================================================================
	public void forwardCheck(Instantiation cpa, long sc) {
		if (sc > this.stepCounter) {
			if (this.updateAgentOrdering(cpa, false)) {
				this.agentView.clearExcept(cpa);
				// this.agentView.priorityMerge(cpa);
				this.agentViewConsistent = true;
			}
			int currDomainSize = this.var.getDomainSize();
			super.forwardCheck(cpa, sc);
			if (this.agentViewConsistent && this.var.getDomainSize() != currDomainSize)
				this.orderMsgH.sendDomainSizeMsgs(cpa);
		}
	}

	// ==========================================================================
	public void processNotOk(Instantiation notOk, long sc) {
		if (sc > this.stepCounter) {
			if (this.updateAgentOrdering(notOk, false)) {
				this.agentView.clearExcept(notOk);
			}
		}
		super.processNotOk(notOk, sc);
	}

	// ==========================================================================
	public boolean updateAgentOrdering(Instantiation cpa, boolean ImNext) {
		IntArray newOrder = cpa.getAgentIds();
		if (ImNext)
			newOrder.add(this.agentId);
		IntArray others = this.agentOrder.getOrder().minus(newOrder);
		newOrder.addAll(others);
		if (this.agentOrder.setNewOrder(newOrder)) {
			this.agentView.ensureOnlyHigherAgentsInView();
			this.constraintHandler.update();
			return true;
		}
		return false;

	}

	// ================================================================================
	// ===============================================================================
	public boolean isMinDom() {
		return this.heuristic == AFCDomDegMaster.MinDom;
	}

	public boolean isDomStaticDeg() {
		return this.heuristic == AFCDomDegMaster.Dom_Static_Deg;
	}

	public boolean isDomDynDeg() {
		return this.heuristic == AFCDomDegMaster.Dom_Dyn_Deg;
	}

	// ===============================================================================

	public int getNextAgent() {
		IntArray assigned = this.agentView.toInstantiation().getAgentIds();
		assigned.add(this.agentId);
		IntArray unassigned = this.agentOrder.getOrder().minus(assigned);
		if (unassigned.size() == 0) { return -1; }
		if (this.isMinDom()) {
			return sortMinDom(unassigned)[0];
		} else {
			return sortMinDomDeg(unassigned)[0];
		}
	}

	// ===============================================================================
	public int[] sortMinDom(IntArray agents) {
		int[] ans = new int[agents.size()];
		for (int i = 0 ; i < ans.length ; i++) {
			int minId = agents.get(0);
			int minSize = domainSizes.get(minId);
			for (int agId : agents) {
				if (minSize > domainSizes.get(agId)) {
					minSize = domainSizes.get(agId);
					minId = agId;
				}
			}
			ans[i] = minId;
			agents.remove(minId);
		}
		return ans;
	}

	public int[] sortMinDomDeg(IntArray agents) {
		int[] ans = new int[agents.size()];
		double minDomDeg, domDeg;
		for (int i = 0 ; i < ans.length ; i++) {
			int minId = agents.get(0);
			int deg = deg(minId, agents);
			if (deg == 0) {
				minDomDeg = Double.POSITIVE_INFINITY;
			} else {
				minDomDeg = ((double) domainSizes.get(minId)) / deg;
			}
			for (int agId : agents) {
				deg = deg(agId, agents);
				if (deg == 0) {
					domDeg = Double.POSITIVE_INFINITY;
				} else {
					domDeg = ((double) domainSizes.get(agId)) / deg;
				}
				if (minDomDeg > domDeg) {
					minDomDeg = domDeg;
					minId = agId;
				}
			}
			ans[i] = minId;
			agents.remove(minId);
		}
		return ans;
	}

	private int deg(int agId, IntArray unassigned) {
		if (this.isDomStaticDeg()) { return this.agentOrder.getAgentsGraph().degree(agId); }
		Iterator<Integer> it = this.agentOrder.getNeighborsOf(agId).iterator();
		int deg = 0;
		while (it.hasNext()) {
			if (unassigned.contains(it.next())) {
				deg++;
			}
		}
		return deg;
	}

	// ==============================================================================

	public class OrderMsgHandler extends MessageHandler {

		public OrderMsgHandler(AbstractAgent agent) {
			super(agent, Commands.ORDER);
		}

		public void processMessage(Message msg) throws StopException {
			int sId = msg.getSenderId();
			int domSize = (Integer) msg.get(Commands.DomainSize);
			long counter = (Long) msg.get(Commands.COUNTER);
			if (counters.containsKey(sId)) {
				if (counters.get(sId) <= counter) {
					counters.put(sId, counter);
					domainSizes.put(sId, domSize);
				}
			} else {
				counters.put(sId, counter);
				domainSizes.put(sId, domSize);
			}
		}

		public void sendDomainSizeMsgs(Instantiation cpa) {
			IntArray agIds = this.myAgent.agentOrder.getOrder();
			IntArray unassigned = agIds.minus(cpa.getAgentIds());
			unassigned.remove(this.myAgent.getAgentId());
			for (int agId : unassigned) {
				this.sendDomainSize(agId);
			}
		}

		public void sendDomainSize(int agId) {
			myCounter++;
			int domSize = getMyVar().getDomainSize();
			Message msg = createMessage();
			msg.addContents(Commands.DomainSize, domSize);
			msg.addContents(Commands.COUNTER, myCounter);
			msg.setReceiverId(agId);
			sendMessage(msg);
		}

	}

	// ==========================================================================================
	public class AFCDomDegAO extends AgentOrdering {
		public AFCDomDegAO(AbstractAgent agent) {
			super(agent);
		}

		public void processOrder(Message msg) {
			super.processOrder(msg);
			for (int agId : this.agentsOrder) {
				domainSizes.put(agId, (Integer) msg.get(agId + ""));
				counters.put(agId, 0L);
			}
		}
	}

}
