/************************************************************
Agile-ABT with termination value as heuristic for changing the 
total order among agents, and using the explanations defined 
as a conjunction of the stored nogoods. 
 *************************************************************/

package simple.dcsp.agileabt.explanation;

import static simple.dcsp.agileabt.explanation.AgileABTMaster.All;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.MinDom;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.Neighbors;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.OrderNeigh;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.OrderOnly;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.Topological_ID;

import java.util.HashMap;
import java.util.Iterator;

import kernel.AbstractAgent;
import kernel.AgentObserver;
import kernel.Communicator;
import kernel.StopException;
import kernel.VarIdentifier;
import kernel.com.Message;
import kernel.events.AgentEventListener;
import kernel.events.VariableEvent;
import kernel.order.AgentOrdering;
import kernel.vars.integer.IntDomainIterator;
import kernel.vars.integer.IntExternalVar;
import kernel.vars.integer.explain.ExplainedIntVar;
import simple.Nogood;
import simple.SingleExplainedVarAgent;
import simple.dcsp.abtfamily.IdleEvent;
import simulation.metric.Metric;
import simulation.metric.MetricListener;
import util.Commands;
import util.Instantiation;
import util.IntArray;
import util.Tools;
import util.VarAssignment;
import util.VarEventMask;

public class AgileABTAgent extends SingleExplainedVarAgent {

	public static final String Explanation = "exp";
	public static final String Order = "order";
	public static final String TV = "TV";
	public static final String LINK = "link";
	public static final String State = "state";
	public static final String EXP_CHECKS = "ExpChecks";
	public static final String NCEXP_CHECKS = "NCExpChecks";
	public static final String CHANGE_TARGET = "ChangeTarget";
	public static final String CHANGE_ORDER = "ChangeOrder";
	public static final String KEEP_TARGET = "KeepTarget";
	public static final String IMPROVE_TV = "ImproveTV";
	public static final String UPDATES_FROM_LHS = "UpdatesFromLhs";
	public static final String IMPROVE_TOPOLOGICAL_TV = "ImproveTopologicalTV";

	private short topologicalType = Topological_ID;

	private short heuristic = MinDom;

	public OkMsgHandler okMsgH = null;
	public NogoodMsgHandler ngdMsgH = null;
	public OrderMsgHandler orderMsgH = null;
	public AddLinkMsgHandler adlMsgH = null;

	public Explanations E;
	public double[] myTV = null;
	private HashMap<Integer, Integer> domains;
	private HashMap<Integer, Integer> degrees;

	public boolean onlyOrder = false;
	public boolean onlyToNeighbors = false;
	public boolean IchangeMyOrder = false;
	public boolean topologicalOrder = false;
	private ExplanationObserver expChecksObserver;

	/**
	 * @param agentId
	 * @param agName
	 * @param description
	 * @param dom
	 * @param type
	 * @param topological
	 * @param topologicalType
	 */
	public AgileABTAgent(int agId, String name, String desc, short heuristic, short type, boolean topological, short topologicalType) {
		super(agId, name, desc);
		this.setType(type);
		this.heuristic = heuristic;
		this.topologicalOrder = topological;
		this.topologicalType = topologicalType;
		this.domains = new HashMap<Integer, Integer>();
		this.degrees = new HashMap<Integer, Integer>();
		this.agentOrder = new AgileABTAO(this);
		this.constructCommunicator();
		// -------------- Messages Handler ------------------//
		this.okMsgH = new OkMsgHandler(this);
		this.ngdMsgH = new NogoodMsgHandler(this);
		this.orderMsgH = new OrderMsgHandler(this);
		this.adlMsgH = new AddLinkMsgHandler(this);
		this.terminateMsgHandler = new AgileABTTermDetection(this);
		this.initAuxStatistics();
		this.expChecksObserver = new ExplanationObserver(NCEXP_CHECKS);
		this.addObserver(this.expChecksObserver);
	}

	private void initAuxStatistics() {
		this.auxiliaryStatistics.put(Commands.nbObsoleteNogood, new Metric(Commands.nbObsoleteNogood));
		this.auxiliaryStatistics.put(Commands.nbAssignments, new Metric(Commands.nbAssignments));
		this.auxiliaryStatistics.put(Commands.ObsoleteOk, new Metric(Commands.ObsoleteOk));
		this.auxiliaryStatistics.put(CHANGE_ORDER, new Metric(CHANGE_ORDER, 0L, Metric.max));
		this.auxiliaryStatistics.put(CHANGE_TARGET, new Metric(CHANGE_TARGET));
		this.auxiliaryStatistics.put(KEEP_TARGET, new Metric(KEEP_TARGET));
		this.auxiliaryStatistics.put(IMPROVE_TV, new Metric(IMPROVE_TV));
		this.auxiliaryStatistics.put(UPDATES_FROM_LHS, new Metric(UPDATES_FROM_LHS));
		this.auxiliaryStatistics.put(IMPROVE_TOPOLOGICAL_TV, new Metric(IMPROVE_TOPOLOGICAL_TV));
		this.auxiliaryStatistics.put(EXP_CHECKS, new Metric(EXP_CHECKS));
		this.auxiliaryStatistics.put(Commands.MSGsSIZE, new Metric(Commands.MSGsSIZE));
		this.auxiliaryStatistics.put(Commands.MaxMSGsSIZE, new Metric(Commands.MaxMSGsSIZE, 0L, Metric.max));
	}

	/**
	 * @param msg
	 */
	public void storeStatisticsFor(Message msg) {
		long size = msg.getSize();
		this.auxiliaryStatistics.get(Commands.MSGsSIZE).value += size;
		if(this.auxiliaryStatistics.get(Commands.MaxMSGsSIZE).value < size){
			this.auxiliaryStatistics.get(Commands.MaxMSGsSIZE).value = size;
		}
	}

	private void setType(short type) {
		switch (type) {
			case All :
				this.onlyToNeighbors = false;
				this.onlyOrder = false;
			break;
			case OrderOnly :
				this.onlyToNeighbors = false;
				this.onlyOrder = true;
			break;
			case Neighbors :
				this.onlyToNeighbors = true;
				this.onlyOrder = false;
			break;
			case OrderNeigh :
				this.onlyToNeighbors = true;
				this.onlyOrder = true;
			break;
		}
	}

	/**
	 * 
	 */
	private void constructCommunicator() {
		this.communicator = new Communicator(this) {
			protected boolean waitForArrivalMessages(long msTimeout) throws InterruptedException {
				logMyDomain();
				this.waitingForMessage = true;
				boolean arrived = false;
				if (((AgileABTAgent) this.myAgent).isIdle()) {
					arrived = mailBoxLock.waitUntilTrue(10);
					if (!arrived) {
						this.myAgent.fireAgentEvent(new IdleEvent(this.myAgent));
					}
				}
				if (!arrived) {
					arrived = mailBoxLock.waitUntilTrue(msTimeout);
				}
				this.waitingForMessage = false;
				logMyDomain();
				return arrived;
			}
		};
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see AbstractAgent#addMsgHandlers()
	 */
	@Override
	public void addMsgHandlers() {
		this.communicator.addMsgHandler(this.okMsgH);
		this.communicator.addMsgHandler(this.ngdMsgH);
		this.communicator.addMsgHandler(this.orderMsgH);
		this.communicator.addMsgHandler(this.adlMsgH);
	}

	@Override
	public void configurePropagationEngine() {
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.INSTANCIATE);
		this.eventsConfig.get(VarEventMask.EXTERNAL_VAR_HIGHER_THAN_LOCAL_VAR).add(VariableEvent.DESINSTANCIATE);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_THE_LAST).add(VariableEvent.RESTORE_VAL);
		this.eventsConfig.get(VarEventMask.LOCAL_VAR_IS_BETWEEN_L_H).add(VariableEvent.RESTORE_VAL);
		this.constraintHandler.update();
	}

	public void initAgentView() {
		IntArray agents = this.getOrder();
		agents.remove(this.agentId);
		for (int agId : agents) {
			if (!this.communicator.isNeighbor(agId)) {
				VarAssignment var = new VarAssignment(new VarIdentifier(agId), Integer.MIN_VALUE);
				IntExternalVar extVar = this.makeExternalVar(var);
				this.agentView.add(extVar);
			} else
				this.agentView.add((IntExternalVar) this.getExternalVar(agId));
		}
	}

	public boolean isIdle() {
		if (this.disSearchStarted) {
			IntArray higherN = this.communicator.getHigherNeighbors();
			Instantiation view = this.agentView.getContextsOfHigherAgents();
			boolean idle = view.getAgentIds().contains(higherN) && this.var.isInstantiated();
			if (idle) {
				view.add(this.var.getVarAssignment());
				((AgileABTTermDetection) this.terminateMsgHandler).SetMyState(this.getOrder(), this.myTV, view);
			}
			return idle;
		}
		return false;
	}

	@Override
	protected void startDistributedSearch() throws StopException {
		this.explanationsConstructor();
		this.addAgentEventListener((AgentEventListener) this.terminateMsgHandler);
		((AgileABTTermDetection) this.terminateMsgHandler).initialize(this.getOrder(), this.myTV);
		this.CheckAgentView();
	}

	// ================================================================================================//
	private void explanationsConstructor() {
		this.E = new Explanations(this.getOrder(), this.domains, this.degrees, this.heuristic);
		this.myTV = E.computeTV(this.getOrder());
	}

	// ================================================================================================//
	/**
	 * 
	 */
	public Explanation getMyExplanation() {
		Instantiation lhs = this.solveMyNogoodStore();
		int rhs = this.var.getDomainSize();
		Explanation myExp = new Explanation(lhs, rhs);
		this.E.setExplanation(this.agentId, myExp);
		return myExp;
	}

	// ================================================================================================//
	/**
	 * @return
	 */
	protected boolean isMyExplanationChanged() {
		if (this.onlyOrder) { return !this.onlyOrder; }
		Explanation oldExp = this.E.getExplanation(this.agentId);
		Explanation newExp = this.getMyExplanation();
		return !oldExp.equals(newExp);
	}

	// ================================================================================================//
	/**
	 * choose an instantiation (i.e a value) compatible with the agentView
	 * 
	 * @throws StopException
	 */
	public void CheckAgentView() throws StopException {
		this.constraintHandler.propagate();
		int value = this.var.getValue();
		if (!this.var.getDomain().contains(value)) {
			if (this.valSelector.hasNextValue()) {
				int myValue = this.valSelector.getNextValue();
				this.var.instantiate(myValue);
				this.okMsgH.sendOkMessages();
			} else {
				this.Backtrack();
			}
		} else {
			if (this.IchangeMyOrder || this.isMyExplanationChanged()) {
				this.okMsgH.sendOkMessages();
			}
		}
	}

	// ================================================================================================//
	/**
	 * Procedure to check if a new link should be added.
	 * 
	 * @param nogood
	 */
	public void CheckAddLink(Instantiation nogood) {
		if (this.onlyToNeighbors) {
			Iterator<VarAssignment> it = nogood.iterator();
			while (it.hasNext()) {
				VarAssignment ass = it.next();
				if (!this.communicator.isNeighbor(ass.getAgentId())) {
					this.communicator.addNeighbor(ass.getAgentId());
					this.agentView.add(this.makeExternalVar(ass));
					this.adlMsgH.sendAddLink(ass);
				}
			}
		}
	}

	// ================================================================================================//
	/**
	 * 
	 */
	private boolean isCompatibleWithMyOrder(int agId, Instantiation instantiation) {
		IntArray predecessors = instantiation.getAgentIds();
		IntArray higherAgents = this.agentOrder.getAgentsHigherThan(agId);
		return higherAgents.contains(predecessors);
	}

	// ================================================================================================//
	/**
	 * If the received explanation is newest take it
	 */
	public boolean updateExplanation(int agId, Explanation exp) {
		boolean viewCompatible = this.agentView.compatibleWith(exp.getLhs());
		boolean orderCompatible = this.isCompatibleWithMyOrder(agId, exp.getLhs());
		if (viewCompatible && orderCompatible) {// && this.agentOrder.isHigher(agId)){
			this.E.setExplanation(agId, exp);
			if (exp.getLhs().isEmpty()) {
				this.E.updateMaxDomainSize(agId, exp.getRhs());
			}
			return true;
		} else {
			this.E.removeExplanationOf(agId);
		}
		return false;
	}

	// ================================================================================================//
	public void updateAgentView(Instantiation inst) {
		Iterator<VarAssignment> it = inst.iterator();
		while (it.hasNext()) {
			VarAssignment varAss = it.next();
			if (this.agentView.compareTimeCounter(varAss) > 0 && this.agentOrder.isHigher(varAss.getAgentId())) {
				this.agentView.priorityMerge(varAss);
				this.auxiliaryStatistics.get(UPDATES_FROM_LHS).value++;
			}
		}
		this.E.removeExplanationsIncompatibleWith(this.agentView.toInstantiation());
	}

	// ================================================================================================//
	public void updateAgentView(Instantiation inst, VarAssignment ass) {
		if (this.agentView.compareTimeCounter(ass) >= 0) {
			this.agentView.priorityMerge(ass);
			this.E.removeExplanationOf(ass.getAgentId());
		} else {
			this.auxiliaryStatistics.get(Commands.ObsoleteOk).value++;
		}
		this.updateAgentView(inst);
	}

	// ================================================================================================//
	private boolean setNewOrder(IntArray order, double[] tv) {
		this.myTV = tv.clone();
		if (this.agentOrder.setNewOrder(order)) {
			this.auxiliaryStatistics.get(CHANGE_ORDER).value++;
			this.IchangeMyOrder = true;
			this.E.refresh(this.agentOrder, this.agentId);
			this.constraintHandler.update();
			return true;
		}
		return false;
	}

	// ================================================================================================//
	/**
	 * @param
	 * @return <!--
	 * 
	 *         <pre>
	 * 		<b><i>true</i></b> if <i>{order,tv}</i> are better than <i>{myOrder,myTV}</i>
	 * 		<b><i>false</i></b> otherwise 
	 * <!--
	 * </pre>
	 */
	private boolean isItBestOrder(IntArray myOrder, double myTV[], IntArray order, double[] tv) {
		int cmp = Tools.compare(myTV, tv);
		if (cmp > 0)
			return false;
		if (cmp < 0)
			return true;
		if (myOrder.compare(order) < 0) {
			return true;
		} else {
			return false;
		}
	}

	// ================================================================================================//
	public boolean checkOrder(IntArray order, double tv[]) {
		if (this.isItBestOrder(this.getOrder(), this.myTV, order, tv)) { return this.setNewOrder(order, tv); }
		return false;
	}

	// ================================================================================================//
	private void removeAssignment(VarAssignment varAss) {
		this.agentView.clear(varAss);
		this.E.remove(varAss);
	}

	protected void incrementNumConstraintChecks(int CCs) {
		for (AgentObserver observer : observers) {
			if (observer instanceof MetricListener) {
				((MetricListener) observer).handleIncrementNbConstraintChecks(CCs);
			}
		}
		this.expChecksObserver.handleIncrementNbConstraintChecks(CCs);
	}

	// ================================================================================================//
	public Explanations updateExplanations(Explanations E, Instantiation ngLhs, VarAssignment x_i, HashMap<Integer, Instantiation> NGs) {
		Explanations E_i = E.clone();
		int nbChecks = E_i.remove(x_i);
		this.expChecksObserver.handleIncrementNbConstraintChecks(nbChecks / 3);
		this.auxiliaryStatistics.get(EXP_CHECKS).value += nbChecks;
		int i = x_i.getAgentId();
		Instantiation lhs_i = new Instantiation();
		lhs_i.priorityMerge(E_i.getLhs(i).clone());
		lhs_i.priorityMerge(ngLhs);
		int rhs_i = E_i.getRhs(i) - 1;
		E_i.setExplanation(i, new Explanation(lhs_i, rhs_i));
		Instantiation myLhs = ngLhs.clone();
		int myRhs = this.getDomainSize(NGs);
		E_i.setExplanation(this.agentId, new Explanation(myLhs, myRhs));
		return E_i;
	}

	// ================================================================================================//
	private HashMap<Integer, Instantiation> removeFrom(HashMap<Integer, Instantiation> NGs, VarAssignment xi) {
		HashMap<Integer, Instantiation> ans = new HashMap<Integer, Instantiation>();
		for (int value : NGs.keySet()) {
			Instantiation ng = NGs.get(value);
			if ((ng.contains(xi.getVarIdentifier()) && ng.compatible(xi)) || !ng.contains(xi.getVarIdentifier())) {
				ans.put(value, ng);
			}
		}
		return ans;
	}

	// ================================================================================================//
	private Explanations chooseVariableOrder(Explanations E, Instantiation nogood, HashMap<Integer, Instantiation> NGs) throws StopException {
		if (nogood.isEmpty()) { throw new StopException(Commands.FAILURE); }
		Explanations selE = null, E_i = null;
		IntArray order_i = null;
		double[] TV_i = this.getMaximumTV(), selTV = this.getMaximumTV();
		VarAssignment x_i = null;
		Instantiation lhs = null, ng_i = null;
		HashMap<Integer, Instantiation> NGs_i = null;
		for (int i = 0 ; i < nogood.size() ; i++) {
			x_i = nogood.get(i);
			int agId = x_i.getAgentId();
			lhs = nogood.clone();
			lhs.remove(x_i);
			NGs_i = this.removeFrom(NGs, x_i);
			E_i = this.updateExplanations(E, lhs, x_i, NGs_i);
			if (E_i.getRhs(agId) == 0) { // It not necessary to backtrack to agentId
				ng_i = E_i.getLhs(agId);
				E_i = this.chooseVariableOrder(E_i, ng_i, NGs_i);
				order_i = E_i.getNewOrder();
				TV_i = E_i.getNewTV();
				ng_i = E_i.getNewNG();
			} else {
				order_i = E_i.getTopologicalOrder(this.topologicalType);
				TV_i = E_i.computeTV(order_i);
				// if (!this.topologicalOrder) {
				// double[] topologicalTV = TV_i.clone();
				// order_i = E_i.getBestOrder();
				// TV_i = E_i.computeTV(order_i);
				// if (Utility.compare(topologicalTV, TV_i) < 0) {
				// this.auxiliaryStatistics.get(IMPROVE_TOPOLOGICAL_TV).value++;
				// }
				// }
				ng_i = nogood.clone();
			}
			if (Tools.compare(selTV, TV_i) < 0) {
				selE = E_i;
				selTV = TV_i;
				selE.setNewOrder(order_i);
				selE.setNewTV(selTV);
				selE.setNewNG(ng_i);
			}
		}
		return selE;
	}

	// ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	public void Backtrack() throws StopException {
		HashMap<Integer, Instantiation> nogoods = this.getNogoods();
		Instantiation nogood = this.solveMyNogoodStore();
		if (nogood.isEmpty()) { throw new StopException(Commands.FAILURE); }
		int target = nogood.last().getAgentId();
		this.E.removeExplanationOf(this.agentId);
		Explanations E = this.chooseVariableOrder(this.E, nogood, nogoods);
		if (Tools.compare(this.myTV, E.getNewTV()) < 0) {
			this.E = E;
			this.setNewOrder(E.getNewOrder(), E.getNewTV());
			Nogood ng = new Nogood(E.getNewNG().clone(this.agentOrder));
			this.ngdMsgH.sendNogood(ng);
			this.removeAssignment(ng.getRhs());
			this.orderMsgH.broadcastOrder(ng.getRhsAgentId());
			if (ng.getRhsAgentId() == target) {
				this.auxiliaryStatistics.get(KEEP_TARGET).value++;
			} else {
				this.auxiliaryStatistics.get(CHANGE_TARGET).value++;
			}
			this.auxiliaryStatistics.get(IMPROVE_TV).value++;
		} else {
			Nogood ng = new Nogood(E.getNewNG().clone(this.agentOrder));
			this.ngdMsgH.sendNogood(ng);
			this.auxiliaryStatistics.get(KEEP_TARGET).value++;
			this.removeAssignment(ng.getRhs());
		}
		this.E.clear();
		this.CheckAgentView();
	}

	private HashMap<Integer, Instantiation> getNogoods() {
		HashMap<Integer, Instantiation> nogoods = new HashMap<Integer, Instantiation>();
		IntDomainIterator it = this.var.getDomain().originalDomainIterator();
		while (it.hasNext()) {
			int value = it.next();
			Instantiation ng = this.makeBetterNogood((ExplainedIntVar) var, value).getLhs();
			nogoods.put(value, ng);
		}
		return nogoods;
	}

	protected int getDomainSize(HashMap<Integer, Instantiation> nogoods) {
		return this.domains.get(this.agentId) - nogoods.size();
	}

	public double[] getMyTV() {
		return this.myTV;
	}

	public void setDom(int agId, int domain) {
		this.domains.put(agId, domain);
	}

	public void setDeg(int agId, int degree) {
		this.degrees.put(agId, degree);
	}

	private double[] getMaximumTV() {
		double[] tv = new double[this.getOrder().size()];
		for (int i = 0 ; i < tv.length ; i++) {
			tv[i] = Integer.MAX_VALUE;
		}
		return tv;
	}

	// =========================================================================================================================/
	public class AgileABTAO extends AgentOrdering {
		public AgileABTAO(AbstractAgent agent) {
			super(agent);
		}

		public void processOrder(Message msg) {
			super.processOrder(msg);
			for (int agId : this.agentsOrder) {
				int domain = (Integer) msg.get(agId + "");
				int degree = this.graph.degree(agId);
				((AgileABTAgent) this.myAgent).setDom(agId, domain);
				((AgileABTAgent) this.myAgent).setDeg(agId, degree);
			}
		}
	}
}
