/**
 * 
 */
package simple.dcsp.agileabt.explanation;

import static simple.dcsp.agileabt.explanation.AgileABTMaster.DomDeg;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.MinDom;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.Topological_Deg;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.Topological_ID;
import static simple.dcsp.agileabt.explanation.AgileABTMaster.Topological_MaxSucc;

import java.util.HashMap;
import java.util.Iterator;

import kernel.order.AgentOrdering;
import util.Instantiation;
import util.IntArray;
import util.VarAssignment;

public class Explanations {

	private HashMap<Integer, Explanation> explanations = null;
	private HashMap<Integer, Integer> maxDomSize = null;
	private HashMap<Integer, Integer> degrees = null;
	private IntArray agentIds = null;

	private IntArray newOrder = null;
	private double[] newTV = null;
	private Instantiation newNG = null;
	private short heuristic = MinDom;

	@SuppressWarnings("unchecked")
	public Explanations(IntArray agentIds, HashMap<Integer, Integer> maxDomSize, HashMap<Integer, Integer> degrees, short heuristic) {
		this.heuristic = heuristic;
		this.agentIds = agentIds.clone();
		this.maxDomSize = (HashMap<Integer, Integer>) maxDomSize.clone();
		this.degrees = degrees;
		this.explanations = new HashMap<Integer, Explanation>();
		for (int agId : agentIds) {
			this.removeExplanationOf(agId);
		}
	}

	public void updateMaxDomainSize(int agId, int d) {
		int value = Math.min(this.getMaxDomainSize(agId), d);
		this.maxDomSize.put(agId, value);
	}

	public int getMaxDomainSize(int agId) {
		return this.maxDomSize.get(agId);
	}

	public void removeExplanationOf(int agId) {
		Instantiation lhs = new Instantiation();
		Integer rhs = this.maxDomSize.get(agId);
		this.setExplanation(agId, new Explanation(lhs, rhs));
	}

	public Instantiation getLhs(int agId) {
		return this.explanations.get(agId).getLhs();
	}

	public int getRhs(int agId) {
		return this.explanations.get(agId).getRhs();
	}

	public Explanation getExplanation(int agId) {
		return this.explanations.get(agId);
	}

	public void setRhs(int agId, int rhs) {
		this.explanations.get(agId).setRhs(rhs);
	}

	public IntArray getNewOrder() {
		return this.newOrder;
	}

	public void setNewOrder(IntArray newOrder) {
		this.newOrder = newOrder;
	}

	public double[] getNewTV() {
		return this.newTV;
	}

	public void setNewTV(double[] newTV) {
		this.newTV = newTV;
	}

	public Instantiation getNewNG() {
		return this.newNG;
	}

	public void setNewNG(Instantiation newNG) {
		this.newNG = newNG;
	}

	public void clear() {
		this.newOrder = null;
		this.newTV = null;
		this.newNG = null;
	}

	@SuppressWarnings("unchecked")
	private HashMap<Integer, Explanation> cloneExplanations() {
		return (HashMap<Integer, Explanation>) this.explanations.clone();
	}

	public void setExplanation(int agId, Explanation exp) {
		this.explanations.put(agId, exp);
	}

	public void setExplanation(int agId, Instantiation lhs, int rhs) {
		this.setExplanation(agId, new Explanation(lhs, rhs));
	}

	/**
	 * remove all explanations incompatible with order
	 */
	public int removeIncompatibleLhsWithOrder(IntArray order) {
		int nbChecks = 0;
		Iterator<Integer> it = this.explanations.keySet().iterator();
		while (it.hasNext()) {
			int agId = it.next();
			IntArray higherAgents = order.getElementsBefor(agId);// this.getHigherAgents(order, agId);
			IntArray predecessors = this.getPredecessors(agId);
			if (!higherAgents.contains(predecessors)) {
				this.removeExplanationOf(agId);
			}
			nbChecks++;
		}
		return nbChecks;
	}

	private IntArray getPredecessors(int agId) {
		return this.getLhs(agId).getAgentIds();
	}

	/**
	 * remove all explanations incompatible with order
	 */
	public void removeExplanationOfAgents(IntArray agents) {
		for (int agId : agents) {
			this.removeExplanationOf(agId);
		}
	}

	/**
	 * 
	 */
	public int refresh(AgentOrdering agentOrder, int myId) {
		return this.removeIncompatibleLhsWithOrder(agentOrder.getOrder());
	}

	public int remove(VarAssignment varAss) {
		int nbCheck = 0;
		Iterator<Integer> it = this.explanations.keySet().iterator();
		while (it.hasNext()) {
			int agId = it.next();
			Explanation exp = this.explanations.get(agId);
			if (exp.getLhs().contains(varAss.getVarIdentifier())) {
				this.removeExplanationOf(agId);
				nbCheck++;
			}
		}
		return nbCheck;
	}

	public IntArray getTopologicalOrder(short topologicalType) {
		IntArray order = new IntArray();
		HashMap<Integer, IntArray> successors = this.getGraphSuccessor();
		HashMap<Integer, IntArray> graph = this.getGraphPredecessor();
		while (graph.keySet().size() > 0) {
			IntArray roots = this.getRoots(graph);
			int selected = -1;
			double min = Integer.MAX_VALUE;
			double rhs = 0;
			switch (topologicalType) {
				case Topological_ID : {
					for (int agId : roots) {
						if (this.heuristic == MinDom) {
							rhs = this.getRhs(agId);
						} else if (this.heuristic == DomDeg) {
							rhs = ((double) this.getRhs(agId)) / this.degrees.get(agId);
						}
						if ((min > rhs) || ((min == rhs) && (agId < selected))) {
							min = this.getRhs(agId);
							selected = agId;
						}
					}
					break;
				}
				case Topological_Deg : {
					int deg = Integer.MIN_VALUE;
					for (int agId : roots) {
						if (this.heuristic == MinDom) {
							rhs = this.getRhs(agId);
						} else if (this.heuristic == DomDeg) {
							rhs = ((double) this.getRhs(agId)) / this.degrees.get(agId);
						}
						if (min > rhs || (min == rhs && (this.degrees.get(agId) > deg))) {
							min = this.getRhs(agId);
							deg = this.degrees.get(agId);
							selected = agId;
						}
					}
					break;
				}
				case Topological_MaxSucc : {
					for (int agId : roots) {
						if (this.heuristic == MinDom) {
							rhs = this.getRhs(agId);
						} else if (this.heuristic == DomDeg) {
							rhs = ((double) this.getRhs(agId)) / this.degrees.get(agId);
						}
						if (min > rhs || (min == rhs && successors.get(agId).size() > successors.get(selected).size())) {
							min = this.getRhs(agId);
							selected = agId;
						}
					}
					break;
				}
			}
			order.add(selected);
			this.remove(graph, selected);
		}
		if (order.size() != this.maxDomSize.keySet().size()) { throw new Error("\nacyclic graph : \n" + this); }
		return order;
	}

	/**
	 * 
	 * @return
	 */
	public IntArray getBestOrder() {
		IntArray order = new IntArray();
		HashMap<Integer, IntArray> graph = this.getGraphPredecessor();
		while (order.size() < this.agentIds.size()) {
			IntArray allRoots = this.getRoots(graph);
			IntArray S1 = this.getMinRoots(allRoots);
			while (S1.size() != 0) {
				if (S1.size() > 1) {
					int d = this.getRhs(S1.get(0));
					IntArray S2 = this.getAgentsWithMinRhs(graph, d);
					if (S2.size() == 0) {
						int selected = S1.get(0);
						order.add(selected);
						this.remove(graph, selected);
						break;
					}
					IntArray S3 = this.getRootsWithMinPredecessor(graph, S2);
					IntArray S4 = this.getMinRoots(S3);
					if (S4.size() == 1) {
						int selected = S4.get(0);
						IntArray predecessors = graph.get(selected);
						for (int pred : graph.get(selected)) {
							predecessors.addAll(graph.get(pred));
						}
						while (predecessors.size() > 0) {
							int pred = predecessors.get(0);
							predecessors.remove(pred);
							if (order.contains(graph.get(pred))) {
								order.add(pred);
								this.remove(graph, pred);
							} else {
								predecessors.add(pred);
							}
						}
						order.add(selected);
						this.remove(graph, selected);
						break;
					} else {
						S1 = S4;
					}
					// S1 = S4;
				} else {
					int selected = S1.get(0);
					order.add(selected);
					this.remove(graph, selected);
					break;
				}
			}
		}
		return order;
	}

	/**
	 * 
	 * @param graph
	 * @param agents
	 * @return
	 */
	private IntArray getRootsWithMinPredecessor(HashMap<Integer, IntArray> graph, IntArray agents) {
		int min = Integer.MAX_VALUE;
		for (int agId : agents) {
			if (graph.get(agId).size() < min) {
				min = graph.get(agId).size();
			}
		}
		IntArray ans = new IntArray();
		for (int agId : agents) {
			if (graph.get(agId).size() == min) {
				ans.add(agId);
			}
		}
		return ans;
	}

	/**
	 * 
	 * @param graph
	 * @param d
	 * @return
	 */
	private IntArray getAgentsWithMinRhs(HashMap<Integer, IntArray> graph, int d) {
		IntArray agents = new IntArray();
		for (int agId : graph.keySet()) {
			if (this.getRhs(agId) < d) {
				agents.add(agId);
				for (int pre : graph.get(agId)) {
					if (this.getRhs(pre) != d) {
						agents.remove(agId);
						break;
					}
				}
			}
		}
		return agents;
	}

	/**
	 * 
	 * @param roots
	 * @return
	 */
	private IntArray getMinRoots(IntArray allRoots) {
		int min = Integer.MAX_VALUE;
		for (int agId : allRoots) {
			if (this.getRhs(agId) < min) {
				min = this.getRhs(agId);
			}
		}
		IntArray roots = new IntArray();
		for (int agId : allRoots) {
			if (this.getRhs(agId) == min) {
				roots.add(agId);
			}
		}
		return roots;
	}

	/**
	 * 
	 * @param graph
	 * @param selected
	 */
	private void remove(HashMap<Integer, IntArray> graph, int selected) {
		for (int agId : graph.keySet()) {
			if (graph.get(agId).contains(selected)) {
				graph.get(agId).remove(selected);
			}
		}
		graph.remove(selected);
	}

	/**
	 * 
	 * @param graph
	 *            is the graph of predecessors
	 * @return
	 */
	private IntArray getRoots(HashMap<Integer, IntArray> graph) {
		IntArray roots = new IntArray();
		for (int agId : graph.keySet()) {
			if ((graph.get(agId).size() == 0)) {
				roots.add(agId);
			}
		}
		return roots;
	}

	/**
	 * 
	 * @return
	 */
	private HashMap<Integer, IntArray> getGraphSuccessor() {
		HashMap<Integer, IntArray> graph = new HashMap<Integer, IntArray>();
		for (int agId : this.agentIds) {
			graph.put(agId, new IntArray());
		}
		for (int agId : this.agentIds) {
			IntArray predecessors = this.getPredecessors(agId);
			for (int pre : predecessors) {
				graph.get(pre).add(agId);
			}
		}
		return graph;
	}

	/**
	 * 
	 * @return
	 */
	private HashMap<Integer, IntArray> getGraphPredecessor() {
		HashMap<Integer, IntArray> graph = new HashMap<Integer, IntArray>();
		for (int agId : this.agentIds) {
			IntArray predecessors = this.getPredecessors(agId);
			graph.put(agId, predecessors);
		}
		return graph;
	}

	/**
	 * 
	 */
	public Explanations clone() {
		IntArray agentIds = this.agentIds.clone();
		Explanations E = new Explanations(agentIds, cloneMaxSizes(), cloneDegrees(), this.heuristic);
		E.explanations = this.cloneExplanations();
		return E;
	}

	public boolean isAcyclic() {
		for (int agId : this.explanations.keySet()) {
			IntArray predecessors = this.getPredecessors(agId);
			for (int pred : predecessors) {
				if (this.getExplanation(pred).getPredecessors().contains(agId)) {
					System.err.println("*******************************************\n" + agId + " :: predecessors" + predecessors + " :: pred: "
							+ pred + "\n" + this);
					return true;
				}
			}
		}
		return false;
	}

	@SuppressWarnings("unchecked")
	public HashMap<Integer, Integer> cloneMaxSizes() {
		return (HashMap<Integer, Integer>) this.maxDomSize.clone();
	}

	@SuppressWarnings("unchecked")
	public HashMap<Integer, Integer> cloneDegrees() {
		return (HashMap<Integer, Integer>) this.degrees.clone();
	}

	public String toString(int agId) {
		Explanation exp = this.getExplanation(agId);
		return agId + " :: " + exp.toString();
	}

	public String toString() {
		String ans = "";
		for (int agId : this.agentIds) {
			ans += this.toString(agId) + "\n";
		}
		return ans;
	}

	public void removeExplanationsIncompatibleWith(Instantiation inst) {
		Iterator<Integer> it = this.explanations.keySet().iterator();
		while (it.hasNext()) {
			int agId = it.next();
			Instantiation lhs = this.getLhs(agId);
			if (!inst.compatible(lhs)) {
				this.removeExplanationOf(agId);
			}
		}
	}

	public double[] computeTV(IntArray order) {
		double[] tv = new double[order.size()];
		for (int i = 0 ; i < order.size() ; i++) {
			int agId = order.get(i);
			if (this.heuristic == MinDom) {
				tv[i] = this.getRhs(agId);
			} else {
				tv[i] = ((double) this.getRhs(agId)) / this.degrees.get(agId);
			}
		}
		return tv;
	}

}
